#=====================================================================
# SQL-Ledger Accounting
# Copyright (C) 2001
#
#  Author: Dieter Simader
#   Email: dsimader@sql-ledger.org
#     Web: http://www.sql-ledger.org
#
#  Contributors:
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#======================================================================
#
# Accounts Payables database backend routines
#
# CHANGE LOG:
#   DS. 2000-07-04  Created
#   DS. 2001-06-12  Changed relations from accno to chart_id
#   DS  2001-07-25  added number parsing
#
#======================================================================


package AP;


sub post_transaction {
  my ($self, $myconfig, $form) = @_;

  # connect to database
  my $dbh = $form->dbconnect_noauto($myconfig);
  
  my $null, $taxrate;
  # split and store id numbers in link accounts
  ($null,$form->{vendor}) = split(/--/, $form->{vendor});
  ($form->{AP}{payables}) = split(/--/, $form->{AP});
  ($form->{AP}{amount}) = split(/--/, $form->{AP_amount});

  # reverse and parse amounts
  $form->{amount} = $form->parse_amount($myconfig, $form->{amount}) * -1;

  foreach my $item (split / /, $form->{taxaccounts}) {
    ($form->{AP}{"tax_$item"}) = $item;
    $form->{"tax_$item"} = $form->parse_amount($myconfig, $form->{"tax_$item"}) * -1;
    $form->{total_tax} += $form->{"tax_$item"};
  }

 
  $form->{taxincluded} = ($form->{taxincluded}) ? TRUE : FALSE;
  if ($form->{taxincluded} eq 'TRUE') {
    # deduct taxes from amount
    $form->{amount} -= $form->{total_tax};
  }
  
  # keep netamount positive
  $form->{netamount} = -$form->{amount};
 
  # set paid to zero if nothing entered
  $form->{paid} = $form->parse_amount($myconfig, $form->{paid});
  
  # store invoice total, this goes into ap table
  $form->{invtotal} = -($form->{amount} + $form->{total_tax});
  
  # amount for total AP
  $form->{payables} = $form->{invtotal};

  # date paid could be empty
  $form->{datepaid} = $form->{transdate} unless ($form->{datepaid});
  

  my $query, $uid, $sth;

  # if we have an id delete old records
  if ($form->{id}) {

    # delete detail records
    $query = qq|DELETE FROM acc_trans WHERE trans_id = $form->{id}|;

    $dbh->do($query) || $form->dberror($query);
    
  } else {
    $uid = time;
    $uid .= $form->{login};
    
    $query = qq|INSERT INTO ap (invnumber) values ( '$uid' )|;
    $dbh->do($query) || $form->dberror($query);

    # retrieve id
    $query = qq|SELECT id
                FROM ap
		WHERE invnumber = '$uid'|;
    $sth = $dbh->prepare($query);
    $sth->execute || $form->dberror($query);

    ($form->{id}) = $sth->fetchrow_array;
    $sth->finish;
    
  }

  my $datepaid = ($form->{paid} != 0) ? qq|'$form->{datepaid}'| : 'NULL';
  
  $query = qq|UPDATE ap SET
	      invnumber = '$form->{invnumber}',
	      transdate = '$form->{transdate}',
	      ordnumber = '$form->{ordnumber}',
	      vendor = $form->{vendor},
	      taxincluded = $form->{taxincluded},
	      amount = $form->{invtotal},
	      duedate = '$form->{duedate}',
	      paid = $form->{paid},
	      datepaid = $datepaid,
	      netamount = $form->{netamount}
	      WHERE id = $form->{id}
	     |;
  $dbh->do($query) || $form->dberror($query);

  # if datepaid == transdate and -payables == paid set payables = 0
  $query = qq|SELECT date '$form->{transdate}' - date '$form->{datepaid}'|;
  $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);
  
  if ((($sth->fetchrow_array)[0] == 0) && ($form->{paid} == $form->{payables})) {
    $form->{payables} = 0;
  }
  $sth->finish;

  
  # add individual transactions
  foreach my $item (keys %{ $form->{AP} }) {
    # insert detail records in acc_trans
    if ($form->{$item} != 0) {
      $query = qq|INSERT INTO acc_trans (trans_id, chart_id, amount, transdate)
                  VALUES ($form->{id}, (SELECT id FROM chart
		                        WHERE accno = $form->{AP}{$item}),
		  $form->{$item}, '$form->{transdate}')|;
      $dbh->do($query) || $form->dberror($query);
    }
  }

  if ($form->{paid} != 0) {
    # get paid account
    ($form->{AP}{paid}) = split(/--/, $form->{AP_paid});
    my @items = ();
    push @items, 'paid';
    push @items, 'payables' if ($form->{payables} != 0);
    
    # add paid and offsetting AP
    $form->{payables} = $form->{paid} * -1;
    
    foreach my $item (@items) {
      $query = qq|INSERT INTO acc_trans (trans_id, chart_id, amount, transdate)
                  VALUES ($form->{id}, (SELECT id FROM chart
		                        WHERE accno = $form->{AP}{$item}),
		  $form->{$item}, '$form->{datepaid}')|;
      $dbh->do($query) || $form->dberror($query);
    }
  }

  
  my $rc = $dbh->commit;
  $dbh->disconnect;

  $rc;
  
}




sub delete_transaction {
  my ($self, $myconfig, $form) = @_;

  # connect to database
  my $dbh = $form->dbconnect_noauto($myconfig);
  
  my $query = qq|DELETE FROM ap WHERE id = $form->{id}|;
  $dbh->do($query) || $form->dberror($query);

  $query = qq|DELETE FROM acc_trans WHERE trans_id = $form->{id}|;
  $dbh->do($query) || $form->dberror($query);
  
  # commit and redirect
  my $rc = $dbh->commit;
  $dbh->disconnect;

  $rc;

}



sub process_payments {
  my ($self, $myconfig, $form) = @_;

  my $dbh = $form->dbconnect_noauto($myconfig);

  my $query;
  
  # go through line by line
  for my $i (1 .. $form->{rowcount}) {
    ($form->{"accno_$i"}) = split(/--/, $form->{"accno_$i"});

    $form->{"paid_$i"} = $form->parse_amount(\%myconfig, $form->{"paid_$i"});
    $form->{"datepaid_$i"} = ($form->{"datepaid_$i"}) ? qq|'$form->{"datepaid_$i"}'| : 'NULL';


    $query = qq|UPDATE ap SET 
 		paid = $form->{"paid_$i"},
		datepaid = $form->{"datepaid_$i"}
		WHERE id = $form->{"id_$i"}|;
    $dbh->do($query) || $form->dberror($query);
    
    # process acc_trans
    if ($form->{"old_paid_$i"} != 0) {
      # update
      $query = qq|UPDATE acc_trans SET
                  chart_id = (SELECT id FROM chart
		              WHERE accno = $form->{"accno_$i"}),
		  amount = $form->{"paid_$i"},
		  transdate = $form->{"datepaid_$i"}
		  WHERE trans_id = $form->{"id_$i"}
		  AND chart_id = (SELECT id FROM chart
		                  WHERE accno = $form->{"old_accno_$i"})|;
      $dbh->do($query) || $form->dberror($query);
    } else {
      if ($form->{"paid_$i"} != 0) {
	# add transaction
	$query = qq|INSERT INTO acc_trans
	            (trans_id, chart_id, amount, transdate)
		    VALUES ($form->{"id_$i"}, (SELECT id FROM chart
		                            WHERE accno = $form->{"accno_$i"}),
		    $form->{"paid_$i"}, $form->{"datepaid_$i"})|;
	$dbh->do($query) || $form->dberror($query);
      }
    }


    # delete AP transactions
    $query = qq|DELETE FROM acc_trans
                WHERE trans_id = $form->{"id_$i"}
		AND chart_id = (SELECT id FROM chart
		                WHERE accno = $form->{"AP_$i"})|;
    $dbh->do($query) || $form->dberror($query);

    # add AP transaction, original
    $query = qq|INSERT INTO acc_trans
		(trans_id, chart_id, amount, transdate)
		VALUES ($form->{"id_$i"}, (SELECT id FROM chart
					   WHERE accno = $form->{"AP_$i"}),
		$form->{"amount_$i"}, '$form->{"transdate_$i"}')|;
    $dbh->do($query) || $form->dberror($query);

    $form->{"paid_$i"} *= -1;
    # add offsetting AP transaction for payment
    $query = qq|INSERT INTO acc_trans
		(trans_id, chart_id, amount, transdate)
		VALUES ($form->{"id_$i"}, (SELECT id FROM chart
					   WHERE accno = $form->{"AP_$i"}),
		$form->{"paid_$i"}, $form->{"datepaid_$i"})|;
    $dbh->do($query) || $form->dberror($query);
    
  }


  $query = qq|DELETE FROM acc_trans WHERE amount = 0|;
  $dbh->do($query) || $form->dberror($query);
  
  $dbh->commit;
  $dbh->disconnect;

}



sub ap_transactions {
  my ($self, $myconfig, $form) = @_;

  # connect to database
  my $dbh = $form->dbconnect($myconfig);
 
  my $query = qq|SELECT ap.id, ap.invnumber, ap.transdate, ap.duedate,
                 ap.amount, ap.paid, ap.ordnumber, vendor.name, ap.invoice,
	         ap.netamount, ap.datepaid
	         FROM ap, vendor
	         WHERE ap.vendor = vendor.id|;
	      
  my $invnumber = lc $form->{invnumber};
  my $ordnumber = lc $form->{ordnumber};
  
  $query .= " AND lower(ap.invnumber) LIKE '%$invnumber%'" if $form->{invnumber};
  $query .= " AND lower(ap.ordnumber) LIKE '%$ordnumber%'" if $form->{ordnumber};
  $query .= " AND ap.vendor = $form->{vendor_id}" if ($form->{vendor_id});
  $query .= " AND ap.transdate >= '$form->{transdatefrom}'" if $form->{transdatefrom};
  $query .= " AND ap.transdate <= '$form->{transdateto}'" if $form->{transdateto};
  $query .= " AND ap.amount <> ap.paid" if ($form->{open} eq "Y");
  $query .= " ORDER by $form->{sort}";
  
  my $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);

  while (my $ap = $sth->fetchrow_hashref) {
    push @{ $form->{AP} }, $ap;
  }
  
  $sth->finish;
  $dbh->disconnect;
  
}


sub payments {
  my ($self, $myconfig, $form) = @_;

  # connect to database
  my $dbh = $form->dbconnect($myconfig);
  
  # get paid accounts from chart
  my $query = qq|SELECT accno, description
                 FROM chart
	         WHERE link LIKE '%AP_paid%'|;
  my $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);

  while (my $ap = $sth->fetchrow_hashref) {
    $form->{paidaccount}{$ap->{accno}} = qq|$ap->{accno}--$ap->{description}|;
  }
  $sth->finish;
    
  # get AP accounts from chart
  $query = qq|SELECT accno
              FROM chart
	      WHERE link = 'AP'
	      ORDER BY accno DESC|;
  $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);

  while ($ap = $sth->fetchrow_hashref) {
    $form->{AP_account}{$ap->{accno}} = $ap->{accno};
  }
  $sth->finish;


  $query = qq|SELECT ap.id, ap.invnumber, ap.transdate, ap.datepaid,
              ap.amount, ap.paid, ap.ordnumber, vendor.name, ap.invoice
	      FROM ap,vendor
	      WHERE ap.vendor = vendor.id|;

  my $invnumber = lc $form->{invnumber};
  my $ordnumber = lc $form->{ordnumber};
  
  $query .= " AND lower(ap.invnumber) LIKE '%$invnumber%'" if $form->{invnumber};
  $query .= " AND lower(ap.ordnumber) LIKE '%$ordnumber%'" if $form->{ordnumber};
  $query .= " AND ap.vendor = $form->{vendor_id}" if ($form->{vendor_id});
  $query .= " AND ap.transdate >= '$form->{transdatefrom}'" if $form->{transdatefrom};
  $query .= " AND ap.transdate <= '$form->{transdateto}'" if $form->{transdateto};
  $query .= " AND ap.amount <> ap.paid" if ($form->{open} eq "Y");
  $query .= " ORDER by $form->{sort}";
  
  $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);


  while ($ap = $sth->fetchrow_hashref) {

    # get accno from acc_trans
    $query = qq|SELECT chart.accno
                FROM acc_trans, chart
                WHERE chart.id = acc_trans.chart_id
		AND trans_id = $ap->{id}|;
    my $ach = $dbh->prepare($query);
    $ach->execute || $form->dberror($query);
    
    while (my $ref = $ach->fetchrow_hashref) {
      # check if the accno matches one of the accounts
      if ($form->{paidaccount}{$ref->{accno}}) {
	$ap->{paidaccno} = $ref->{accno};
      }
      if ($form->{AP_account}{$ref->{accno}}) {
	$ap->{AP_account} = $ref->{accno};
      }
    }
    $ach->finish;

    push @{ $form->{AP} }, $ap;

  }
  
  $sth->finish;
  $dbh->disconnect;

}

1;

