#=====================================================================
# SQL-Ledger Accounting
# Copyright (C) 2001
#
#  Author: Dieter Simader
#   Email: dsimader@sql-ledger.org
#     Web: http://www.sql-ledger.org
#
#  Contributors:
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#======================================================================
#
# Accounts Receivable module backend routines
#
# CHANGE LOG:
#   DS. 2000-07-04  Created
#   DS. 2001-07-25  added number parsing
#
#======================================================================

package AR;


sub post_transaction {
  my ($self, $myconfig, $form) = @_;

  my $null, $taxrate;
  # split and store id numbers in link accounts
  ($null,$form->{customer}) = split(/--/, $form->{customer});
  ($form->{AR}{receivables}) = split(/--/, $form->{AR});
  ($form->{AR}{amount}) = split(/--/, $form->{AR_amount});
  
  
  $form->{amount} = $form->parse_amount($myconfig, $form->{amount});
  
  foreach my $item (split / /, $form->{taxaccounts}) {
    $form->{AR}{"tax_$item"} = $item;
    $form->{"tax_$item"} = $form->parse_amount($myconfig, $form->{"tax_$item"});
    $form->{total_tax} += $form->{"tax_$item"};
  }


  $form->{taxincluded} = ($form->{taxincluded}) ? TRUE : FALSE;
  
  if ($form->{taxincluded} eq 'TRUE') {
    $form->{amount} -= $form->{total_tax};
  }
  
  # set amounts to 0 if there is no value
  $form->{paid} = $form->parse_amount($myconfig, $form->{paid});
  
  # store invoice total, this goes into ar table
  $form->{invtotal} = $form->{amount} + $form->{total_tax};
  $form->{invpaid} = $form->{paid};
  
  # amount for AR account
  $form->{receivables} = $form->{invtotal} * -1;

  # reverse paid
  $form->{paid} *= -1;
  
  $form->{datepaid} = $form->{transdate} unless $form->{datepaid};

  # connect to database
  my $dbh = $form->dbconnect_noauto($myconfig);

  my $query, $uid, $sth;
  
  # if we have an id delete old records
  if ($form->{id}) {

    # delete detail records
    $query = qq|DELETE FROM acc_trans WHERE trans_id = $form->{id}|;
    $dbh->do($query) || $form->dberror($query);
    
  } else {
    $uid = time;
    $uid .= $form->{login};

    $query = qq|INSERT INTO ar (invnumber) values ('$uid')|;
    $dbh->do($query) || $form->dberror($query);

    # retrieve id
    $query = qq|SELECT id
                FROM ar
		WHERE invnumber = '$uid'|;
    $sth = $dbh->prepare($query);
    $sth->execute || $form->dberror($query);

    ($form->{id}) = $sth->fetchrow_array;
    $sth->finish;
    
  }

  my $datepaid = ($form->{paid} != 0) ? qq|'$form->{datepaid}'| : 'NULL';
  
  $query = qq|UPDATE ar set
	      invnumber = '$form->{invnumber}',
	      transdate = '$form->{transdate}',
	      customer = $form->{customer},
	      taxincluded = $form->{taxincluded},
	      amount = $form->{invtotal},
	      duedate = '$form->{duedate}',
	      paid = $form->{invpaid},
	      datepaid = $datepaid,
	      netamount = $form->{amount}
	      WHERE id = $form->{id}
	     |;
  $dbh->do($query) || $form->dberror($query);

  # if transdate == datepaid  and receivables = 0 set to 0
  $query = qq|SELECT date '$form->{transdate}' - date '$form->{datepaid}'|;
  $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);
  
  if ((($sth->fetchrow_array)[0] == 0) && ($form->{paid} == $form->{receivables})) {
    $form->{receivables} = 0;
  }
  $sth->finish;
  
  
  # add individual transactions
  foreach my $item (keys %{ $form->{AR} }) {

    if ($form->{$item}) {
      # insert detail records in acc_trans
      $query = qq|INSERT INTO acc_trans (trans_id, chart_id, amount, transdate)
		  VALUES ($form->{id}, (SELECT id FROM chart
		                        WHERE accno = $form->{AR}{$item}),
		  $form->{$item}, '$form->{transdate}')|;
      $dbh->do($query) || $form->dberror($query);
    }
  }

  if ($form->{paid} != 0) {
    ($form->{AR}{paid}) = split(/--/, $form->{AR_paid});
    my @items = ();
    push @items, 'paid';
    push @items, 'receivables' if ($form->{receivables} != 0);
    
    $form->{receivables} = $form->{paid} * -1;
    
    foreach my $item (@items) {
      $query = qq|INSERT INTO acc_trans (trans_id, chart_id, amount, transdate)
		  VALUES ($form->{id}, (SELECT id FROM chart
					WHERE accno = $form->{AR}{$item}),
		  $form->{$item}, '$form->{datepaid}')|;
      $dbh->do($query) || $form->dberror($query);
    }
  }


  my $rc = $dbh->commit;
  $dbh->disconnect;
  
  $rc;
  
}



sub delete_transaction {
  my ($self, $myconfig, $form) = @_;

  # connect to database, turn AutoCommit off
  my $dbh = $form->dbconnect_noauto($myconfig);

  my $query = qq|DELETE FROM ar WHERE id = $form->{id}|;
  $dbh->do($query) || $form->dberror($query);

  $query = qq|DELETE FROM acc_trans WHERE trans_id = $form->{id}|;
  $dbh->do($query) || $form->dberror($query);
  
  # commit
  my $rc = $dbh->commit;
  $dbh->disconnect;
  
  $rc;
  
}



sub process_receipts {
  my ($self, $myconfig, $form) = @_;
    
  # connect to database, turn AutoCommit off
  my $dbh = $form->dbconnect_noauto($myconfig);

  my $query;

  # go through line by line and compare
  for my $i (1 .. $form->{rowcount}) {
    ($form->{"accno_$i"}) = split(/--/, $form->{"accno_$i"});

    $form->{"paid_$i"} = $form->parse_amount($myconfig, $form->{"paid_$i"});
    
    $form->{"datepaid_$i"} = ($form->{"datepaid_$i"}) ? qq|'$form->{"datepaid_$i"}'| : 'NULL';

    $query = qq|UPDATE ar SET 
 		paid = $form->{"paid_$i"},
		datepaid = $form->{"datepaid_$i"}
		WHERE id = $form->{"id_$i"}|;
    $dbh->do($query) || $form->dberror($query);
    
    
    # reverse amounts paid
    $form->{"paid_$i"} *= -1;
    $form->{"old_paid_$i"} *= -1;
    $form->{"amount_$i"} *= -1;
    
    # update or add paid transaction
    if ($form->{"old_paid_$i"} != 0) {
      # update
      $query = qq|UPDATE acc_trans SET
      		  chart_id = (SELECT id FROM chart
		              WHERE accno = $form->{"accno_$i"}),
                  amount = $form->{"paid_$i"},
		  transdate = $form->{"datepaid_$i"}
		  WHERE trans_id = $form->{"id_$i"}
		  AND chart_id = (SELECT id FROM chart
		                  WHERE accno = $form->{"old_accno_$i"})|;
      $dbh->do($query) || $form->dberror($query);
      
    } else {
      if ($form->{"paid_$i"} != 0) {
	# add transaction
	$query = qq|INSERT INTO acc_trans
	            (trans_id, chart_id, amount, transdate)
		    VALUES ($form->{"id_$i"}, (SELECT id FROM chart
		                           WHERE accno = $form->{"accno_$i"}),
		    $form->{"paid_$i"}, $form->{"datepaid_$i"})|;
	$dbh->do($query) || $form->dberror($query);
      }
    }

    # delete AR transactions
    $query = qq|DELETE FROM acc_trans
                WHERE trans_id = $form->{"id_$i"}
		AND chart_id = (SELECT id FROM chart
		                WHERE accno = $form->{"AR_$i"})|;
    $dbh->do($query) || $form->dberror($query);


    # add AR transaction, this is the original
    $query = qq|INSERT INTO acc_trans
		(trans_id, chart_id, amount, transdate)
		VALUES ($form->{"id_$i"}, (SELECT id FROM chart
				       WHERE accno = $form->{"AR_$i"}),
		$form->{"amount_$i"}, '$form->{"transdate_$i"}')|;
    $dbh->do($query) || $form->dberror($query);

    $form->{"paid_$i"} *= -1;
    # add AR transaction, this is the offsetting AR for the receipt
    $query = qq|INSERT INTO acc_trans
		(trans_id, chart_id, amount, transdate)
		VALUES ($form->{"id_$i"}, (SELECT id FROM chart
				       WHERE accno = $form->{"AR_$i"}),
		$form->{"paid_$i"}, $form->{"datepaid_$i"})|;
    $dbh->do($query) || $form->dberror($query);

  }

  $query = qq|DELETE FROM acc_trans WHERE amount = 0|;
  $dbh->do($query) || $form->dberror($query);
    
  $dbh->commit;
  $dbh->disconnect;

}



sub ar_transactions {
  my ($self, $myconfig, $form) = @_;

  # connect to database
  my $dbh = $form->dbconnect($myconfig);

  my $query = qq|SELECT ar.id, ar.invnumber, ar.transdate, ar.duedate,
                 ar.netamount, ar.amount, ar.paid, customer.name, ar.invoice,
 	         ar.datepaid, ar.terms
	         FROM ar, customer
	         WHERE ar.customer = customer.id|;
  
  my $invnumber = lc $form->{invnumber};
    
  $query .= " AND lower(ar.invnumber) LIKE '%$invnumber%'" if $form->{invnumber};
  $query .= " AND ar.customer = $form->{customer_id}" if ($form->{customer_id});
  $query .= " AND ar.transdate >= '$form->{transdatefrom}'" if $form->{transdatefrom};
  $query .= " AND ar.transdate <= '$form->{transdateto}'" if $form->{transdateto};
  $query .= " AND ar.amount <> ar.paid" if ($form->{open} eq "Y");
  $query .= " ORDER by $form->{sort}";
  
  my $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);

  while (my $ar = $sth->fetchrow_hashref) {
    push @{ $form->{AR} }, $ar;
  }
  
  $sth->finish;
  $dbh->disconnect;

}



sub receipts {
  my ($self, $myconfig, $form) = @_;

  # connect to database
  my $dbh = $form->dbconnect($myconfig);

  # get paid accounts from chart
  my $query = qq|SELECT accno, description
                 FROM chart
	         WHERE link LIKE '%AR_paid%'|;
  my $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);

  my $ar;
  while ($ar = $sth->fetchrow_hashref) {
    $form->{paidaccount}{$ar->{accno}} = qq|$ar->{accno}--$ar->{description}|;
  }
  $sth->finish;
    
  # get AR accounts from chart
  $query = qq|SELECT accno
              FROM chart
	      WHERE link = 'AR'
	      ORDER BY accno DESC|;
  $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);

  while ($ar = $sth->fetchrow_hashref) {
    $form->{AR_account}{$ar->{accno}} = $ar->{accno};
  }
  $sth->finish;
    
	      
  $query = qq|SELECT ar.id, ar.invnumber, ar.transdate, ar.datepaid,
              ar.amount, ar.paid, customer.name, ar.invoice
	      FROM ar,customer
	      WHERE ar.customer = customer.id|;

  my $invnumber = lc $form->{invnumber};
  
  $query .= " AND lower(ar.invnumber) LIKE '%$invnumber%'" if $form->{invnumber};
  $query .= " AND ar.customer = $form->{customer_id}" if $form->{customer_id};
  $query .= " AND ar.transdate >= '$form->{transdatefrom}'" if $form->{transdatefrom};
  $query .= " AND ar.transdate <= '$form->{transdateto}'" if $form->{transdateto};
  $query .= " AND ar.amount <> ar.paid" if ($form->{open} eq "Y");
  $query .= " ORDER by $form->{sort}";
  
  $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);

  
  while ($ar = $sth->fetchrow_hashref) {

    # get accno from acc_trans
    $query = qq|SELECT accno
                FROM chart
		WHERE chart.id = acc_trans.chart_id
		AND acc_trans.trans_id = $ar->{id}|;
    my $ach = $dbh->prepare($query);
    $ach->execute || $form->dberror($query);
    
    while (my $ref = $ach->fetchrow_hashref) {
      # check if the accno matches one of the accounts
      if ($form->{paidaccount}{$ref->{accno}}) {
	$ar->{paidaccno} = $ref->{accno};
      }
      if ($form->{AR_account}{$ref->{accno}}) {
	$ar->{AR_account} = $ref->{accno};
      }
    }
    $ach->finish;

    push @{ $form->{AR} }, $ar;

  }
  
  $sth->finish;
  $dbh->disconnect;

}


1;

