#=====================================================================
# SQL-Ledger Accounting
# Copyright (C) 2001
#
#  Author: Dieter Simader
#   Email: dsimader@sql-ledger.org
#     Web: http://www.sql-ledger.org
#
#  Contributors:
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#======================================================================
#
# General ledger backend code
#
# CHANGE LOG:
#   DS. 2000-07-04  Created
#   DS. 2001-06-12  Changed relations from accno to chart_id
#
#======================================================================

package GL;


sub delete_transaction {
  my ($self, $myconfig, $form) = @_;
  
  # connect to database
  my $dbh = $form->dbconnect_noauto($myconfig);

  my $query = qq|DELETE FROM gl WHERE id = $form->{id}|;
  $dbh->do($query) || $form->dberror($query);

  $query = qq|DELETE FROM acc_trans WHERE trans_id = $form->{id}|;
  $dbh->do($query) || $form->dberror($query);

  # commit and redirect
  my $rc = $dbh->commit;
  $dbh->disconnect;
  
  $rc;
  
}


sub post_transaction {
  my ($self, $myconfig, $form) = @_;
  
  my $debit, $credit = 0;

  my $i;
  # check if debit and credit balances
  for $i (1 .. $form->{rowcount}) {
    if ($form->{"debit_$i"} && $form->{"credit_$i"}) {
      return -1;
    }

    $form->{"debit_$i"} = $form->parse_amount($myconfig, $form->{"debit_$i"});
    $form->{"credit_$i"} = $form->parse_amount($myconfig, $form->{"credit_$i"});

    $debit += $form->{"debit_$i"};
    $credit += $form->{"credit_$i"};
  }

  $debit = sprintf("%.2f", $debit);
  $credit = sprintf("%.2f", $credit);
    
  if ($debit != $credit) {
    return -2;
  }

  # connect to database, turn off AutoCommit
  my $dbh = $form->dbconnect_noauto($myconfig);

  # post the transaction
  # make up a unique handle and store in source field
  # then retrieve the record based on the unique handle to get the id
  # replace the source field with the actual variable
  # add records to acc_trans

  # if there is a $form->{id} replace the old transaction
  # delete all acc_trans entries and add the new ones

  # escape '
  $form->{source} =~ s/'/\\'/g;
  $form->{description} =~ s/'/\\'/g;


  my $query, $uid, $sth;
  
  if ($form->{id}) {
    $query = qq|UPDATE gl SET 
  		source = '$form->{source}',
		description = '$form->{description}',
		transdate = '$form->{transdate}'
	        WHERE id = $form->{id}|;
	     
  } else {
    $uid = time;
    $uid .= $form->{login};

    $query = qq|INSERT INTO gl (source,description,transdate)
		VALUES ('$uid', '$form->{description}', '$form->{transdate}')|;
  }

  $dbh->do($query) || $form->dberror($query);


  if ($form->{id}) {
    # delete individual transactions
    $query = qq|DELETE FROM acc_trans 
                WHERE trans_id = $form->{id}|;
    $dbh->do($query) || $form->dberror($query);
  } else {
    
    $query = qq|SELECT id FROM gl WHERE source = '$uid'|;
    $sth = $dbh->prepare($query);
    $sth->execute || $form->dberror($query);

    ($form->{id}) = $sth->fetchrow_array;
    $query = qq|UPDATE gl SET source = '$form->{source}'
                WHERE id = $form->{id}|;
    $dbh->do($query) || $form->dberror($query);

    $sth->finish;
  }
  
  
  # insert acc_trans transactions
  for $i (1 .. $form->{rowcount}) {
    # extract accno
    ($accno) = split(/--/, $form->{"accno_$i"});
    my $amount = 0;
    
    if ($form->{"credit_$i"} != 0) {
      $amount = $form->{"credit_$i"};
    }
    if ($form->{"debit_$i"} != 0) {
      $amount = $form->{"debit_$i"} * -1;
    }


    # if there is an amount, add the record
    if ($amount != 0) {
      $query = qq|INSERT INTO acc_trans (trans_id, chart_id, amount, transdate)
		  VALUES
		  ($form->{id}, (SELECT id
		                 FROM chart
				 WHERE accno = $accno),
		   $amount, '$form->{transdate}' )|;
    
      $dbh->do($query) || $form->dberror($query);
    }
  }

  # commit and redirect
  my $rc = $dbh->commit;
  $dbh->disconnect;

  $rc;

}



sub all_transactions {
  my ($self, $myconfig, $form) = @_;

  # connect to database
  my $dbh = $form->dbconnect($myconfig);

  my $glwhere = TRUE;
  my $arwhere = TRUE;
  my $apwhere = TRUE;
  
  if ($form->{source}) {
    my $source = lc $form->{source};
    $glwhere .= " AND lower(source) LIKE '%$source%'";
    $arwhere .= " AND lower(ar.invnumber) LIKE '%$source%'";
    $apwhere .= " AND lower(ap.invnumber) LIKE '%$source%'";
  }
  if ($form->{datefrom}) {
    $glwhere .= " AND acc_trans.transdate >= '$form->{datefrom}'";
    $arwhere .= " AND acc_trans.transdate >= '$form->{datefrom}'";
    $apwhere .= " AND acc_trans.transdate >= '$form->{datefrom}'";
  }
  if ($form->{dateto}) {
    $glwhere .= " AND acc_trans.transdate <= '$form->{dateto}'";
    $arwhere .= " AND acc_trans.transdate <= '$form->{dateto}'";
    $apwhere .= " AND acc_trans.transdate <= '$form->{dateto}'";
  }
  if ($form->{description}) {
    my $description = lc $form->{description};
    $glwhere .= " AND lower(gl.description) LIKE '%$description%'";
    $arwhere .= " AND lower(customer.name) LIKE '%$description%'";
    $apwhere .= " AND lower(vendor.name) LIKE '%$description%'";
  }
  if ($form->{accno}) {
    $glwhere .= " AND chart.accno = $form->{accno}";
    $arwhere .= " AND chart.accno = $form->{accno}";
    $apwhere .= " AND chart.accno = $form->{accno}";
  }
  if ($form->{category} ne 'X') {
    $glwhere .= " AND chart.category = '$form->{category}'";
    $arwhere .= " AND chart.category = '$form->{category}'";
    $apwhere .= " AND chart.category = '$form->{category}'";
  }

  my $query = qq|SELECT gl.id, 'gl' AS type, FALSE AS invoice, gl.source,
                 gl.description, acc_trans.transdate, chart.accno,
                 (CASE WHEN acc_trans.amount < 0 THEN -acc_trans.amount END)
		   AS debit,
		 (CASE WHEN acc_trans.amount > 0 THEN acc_trans.amount END)
		   AS credit
                 FROM gl, acc_trans, chart
                 WHERE $glwhere
		 AND acc_trans.chart_id = chart.id
		 AND gl.id = acc_trans.trans_id
	UNION
	         SELECT ar.id, 'ar' AS type, ar.invoice, ar.invnumber,
		 customer.name, acc_trans.transdate, chart.accno,
		 (CASE WHEN acc_trans.amount < 0 THEN -acc_trans.amount END)
		   AS debit,
		 (CASE WHEN acc_trans.amount > 0 THEN acc_trans.amount END)
		   AS credit
		 FROM ar, acc_trans, chart
		 WHERE $arwhere
		 AND acc_trans.chart_id = chart.id
		 AND ar.customer = customer.id
		 AND ar.id = acc_trans.trans_id
	UNION
	         SELECT ap.id, 'ap' AS type, ap.invoice, ap.invnumber,
		 vendor.name, acc_trans.transdate, chart.accno,
		 (CASE WHEN acc_trans.amount < 0 THEN -acc_trans.amount END)
		   AS debit,
		 (CASE WHEN acc_trans.amount > 0 THEN acc_trans.amount END)
		   AS credit
		 FROM ap, acc_trans, chart
		 WHERE $apwhere
		 AND acc_trans.chart_id = chart.id
		 AND ap.vendor = vendor.id
		 AND ap.id = acc_trans.trans_id
	         ORDER BY $form->{sort}|;
  my $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);

  while (my $ref = $sth->fetchrow_hashref) {

    # gl
    if ($ref->{type} eq "gl") {
      $ref->{module} = "gl";
    }

    # ap
    if ($ref->{type} eq "ap") {
      if ($ref->{invoice}) {
        $ref->{module} = "ir";
      } else {
        $ref->{module} = "ap";
      }
    }

    # ar
    if ($ref->{type} eq "ar") {
      if ($ref->{invoice}) {
        $ref->{module} = "is";
      } else {
        $ref->{module} = "ar";
      }
    }

    push @{ $form->{GL} }, $ref;
    
  }

  $sth->finish;

  if ($form->{accno}) {
    $query = qq|SELECT description FROM chart WHERE accno = $form->{accno}|;
    $sth = $dbh->prepare($query);
    $sth->execute || $form->dberror($query);

    ($form->{account_description}) = $sth->fetchrow_array;
    $sth->finish;
  }
  
  $dbh->disconnect;

}


sub transaction {
  my ($self, $myconfig, $form) = @_;
  
  my $query, $sth;
  
  # connect to database
  my $dbh = $form->dbconnect($myconfig);

  if ($form->{id}) {
    $query = "SELECT source, description, transdate
              FROM gl
	      WHERE id = $form->{id}";
    $sth = $dbh->prepare($query);
    $sth->execute || $form->dberror($query);

    ($form->{source}, $form->{description}, $form->{transdate}) = $sth->fetchrow_array;
    $sth->finish;
  
    # retrieve individual rows
    $query = "SELECT chart.accno, amount
	      FROM acc_trans, chart
	      WHERE acc_trans.chart_id = chart.id
	      AND trans_id = $form->{id}
	      ORDER BY accno";
    $sth = $dbh->prepare($query);
    $sth->execute || $form->dberror($query);
    
    while (my $ref = $sth->fetchrow_hashref) {
      push @{ $form->{GL} }, $ref;
    }
  } else {
    $query = "SELECT current_date";
    $sth = $dbh->prepare($query);
    $sth->execute || $form->dberror($query);

    ($form->{transdate}) = $sth->fetchrow_array;
  }

  $sth->finish;

  # get chart of accounts
  $query = qq|SELECT accno,description
              FROM chart
	      WHERE charttype = 'A'
              ORDER by accno|;
  $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);
  $form->{chart} = "";
  while (my $ref = $sth->fetchrow_hashref) {
    $form->{chart} .= "$ref->{accno}--$ref->{description}\n";
  }
  $sth->finish;
  
  $dbh->disconnect;

}


1;

