#=====================================================================
# SQL-Ledger Accounting
# Copyright (C) 2001
#
#  Author: Dieter Simader
#   Email: dsimader@sql-ledger.org
#     Web: http://www.sql-ledger.org
#
#  Contributors: Antonio Gallardo <agssa@ibw.com.ni>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#======================================================================
#
# backend code for reports
# 
# CHANGE LOG:
#   DS. 2000-07-04  Created
#   AG. 2001-04-16  added dash_amount function to format numbers
#                   added trial_balance_details function
#   DS. 2001-06-12  Changed relations from accno to chart_id
#   BL. 2001-06-17  aging reports
#   DS. 2001-07-22  removed dash_amount
#                   replaced by $form->format_amount which also
#                   formats numbers with dot and commas
#
#======================================================================

package RP;


sub income_statement {
  my ($self, $myconfig, $form) = @_;

  # connect to database
  my $dbh = $form->dbconnect($myconfig);

  my $last_period = 0;
  my $where = 'true';

  # if there are any dates construct a where
  if ($form->{fromdate} || $form->{todate}) {
    if ($form->{fromdate}) {
      $where .= " AND transdate >= '$form->{fromdate}'";
    }
    if ($form->{todate}) {
      $where .= " AND transdate <= '$form->{todate}'";
    }
  }

  foreach my $key (qw(I E)) {
    &get_accounts($dbh, $last_period, $where, $key, $form);
  }
  
  $where = 'true';
  # if there are any compare dates
  if ($form->{comparefromdate} || $form->{comparetodate}) {
    if ($form->{comparefromdate}) {
      $where .= " AND transdate >= '$form->{comparefromdate}'";
    }
    if ($form->{comparetodate}) {
      $where .= " AND transdate <= '$form->{comparetodate}'";
    }

    $last_period = 1;

    foreach $key (qw(I E)) {
      &get_accounts($dbh, $last_period, $where, $key, $form);
    }
  }  

  
  # disconnect
  $dbh->disconnect;


  # now we got $form->{I}{accno}{ }
  # and $form->{E}{accno}{  }
  # build income accounts
  
  foreach $key (sort keys %{ $form->{I} }) {
    # push description onto array
    push(@{$form->{income_account}}, "$form->{I}{$key}{description}");
    
    # keep running total
    $form->{total_income_this_period} += $form->{I}{$key}{this};
    
    # add amount or a -
    push(@{$form->{income_this_period}}, $form->format_amount($myconfig, $form->{I}{$key}{this}, 1));
    
    # add amount or - for last period
    if ($last_period) {
      $form->{total_income_last_period} += $form->{I}{$key}{last};

      push(@{$form->{income_last_period}}, $form->format_amount($myconfig,$form->{I}{$key}{last}, 1));
    }
  }


  # build expense accounts
  foreach $key (sort keys %{ $form->{E} }) {
    push(@{$form->{expense_account}}, "$form->{E}{$key}{description}");
    
    $form->{total_expenses_this_period} += ($form->{E}{$key}{this} * -1);

    push(@{$form->{expenses_this_period}}, $form->format_amount($myconfig, $form->{E}{$key}{this} * -1, 1));
    
    if ($last_period) {
      $form->{total_expenses_last_period} += ($form->{E}{$key}{last} * -1);

      push(@{$form->{expenses_last_period}}, $form->format_amount($myconfig, $form->{E}{$key}{last} * -1, 1));
    }
  }

  
  # totals for income and expenses
  $form->{total_income_this_period} = sprintf("%.2f", $form->{total_income_this_period});
  $form->{total_expenses_this_period} = sprintf("%.2f", $form->{total_expenses_this_period});

  # total for income/loss
  $form->{total_this_period} = $form->{total_income_this_period} - $form->{total_expenses_this_period};
  
  if ($last_period) {
    # total for income/loss
    $form->{total_last_period} = $form->format_amount($myconfig, $form->{total_income_last_period} - $form->{total_expenses_last_period}, 1);
    
    # totals for income and expenses for last_period
    $form->{total_income_last_period} = $form->format_amount($myconfig, $form->{total_income_last_period}, 1);
    $form->{total_expenses_last_period} = $form->format_amount($myconfig, $form->{total_expenses_last_period}, 1);

  }


  $form->{total_income_this_period} = $form->format_amount($myconfig,$form->{total_income_this_period}, 1);
  $form->{total_expenses_this_period} = $form->format_amount($myconfig,$form->{total_expenses_this_period}, 1);
  $form->{total_this_period} = $form->format_amount($myconfig,$form->{total_this_period}, 1);

}


sub balance_sheet {
  my ($self, $myconfig, $form) = @_;
  
  # connect to database
  my $dbh = $form->dbconnect($myconfig);

  my $last_period = 0;
  my $where = 'true';

  # if there are any dates construct a where
  if ($form->{asofdate}) {
    $where .= " AND transdate <= '$form->{asofdate}'";
    
    $form->{this_period} = "$form->{asofdate}";
    $form->{period} = "$form->{asofdate}";
    
  }

  foreach my $key (qw(A L Q)) {
    &get_accounts($dbh, $last_period, $where, $key, $form);
  }
  
  $where = 'true';
  # if there are any compare dates
  if ($form->{compareasofdate}) {
    $where .= " AND transdate <= '$form->{compareasofdate}'";

    $last_period = 1;
    foreach $key (qw(A L Q)) {
      &get_accounts($dbh, $last_period, $where, $key, $form);
    }
  
    $form->{last_period} = "$form->{compareasofdate}";

  }  

  
  # disconnect
  $dbh->disconnect;


  # now we got $form->{A}{accno}{ }    assets
  # and $form->{L}{accno}{ }           liabilities
  # and $form->{Q}{accno}{ }           equity
  # build asset accounts
  
  foreach $key (sort keys %{ $form->{A} }) {
    # push description onto array
    push(@{$form->{asset_account}}, "$form->{A}{$key}{description}");
    
    $form->{total_assets_this_period} += $form->{A}{$key}{this} * -1;

    push(@{$form->{asset_this_period}}, $form->format_amount($myconfig, $form->{A}{$key}{this} * -1, 1));
    
    if ($last_period) {
      $form->{total_assets_last_period} += $form->{A}{$key}{last} * -1;

      push(@{$form->{asset_last_period}}, $form->format_amount($myconfig, $form->{A}{$key}{last} * -1, 1));
    }
  }


  # build liability accounts
  foreach $key (sort keys %{ $form->{L} }) {
    push(@{$form->{liability_account}}, "$form->{L}{$key}{description}");
    $form->{total_liabilities_this_period} += $form->{L}{$key}{this};

    push(@{$form->{liability_this_period}}, $form->format_amount($myconfig, $form->{L}{$key}{this}, 1));
    
    if ($last_period) {
      $form->{total_liabilities_last_period} += $form->{L}{$key}{last};

      push(@{$form->{liability_last_period}}, $form->format_amount($myconfig, $form->{L}{$key}{last}, 1));
    }
  }


  # build equity accounts
  foreach $key (sort keys %{ $form->{Q} }) {
    push(@{$form->{equity_account}}, "$form->{Q}{$key}{description}");
    $form->{total_equity_this_period} += $form->{Q}{$key}{this};

    push(@{$form->{equity_this_period}}, $form->format_amount($myconfig, $form->{Q}{$key}{this}, 1));
    
    if ($last_period) {
      $form->{total_equity_last_period} += $form->{Q}{$key}{last};

      push(@{$form->{equity_last_period}}, $form->format_amount($myconfig, $form->{Q}{$key}{last}, 1));
    }
  }

  
  # totals for assets, liabilities
  $form->{total_assets_this_period} = sprintf("%.2f", $form->{total_assets_this_period});
  $form->{total_liabilities_this_period} = sprintf("%.2f", $form->{total_liabilities_this_period});
  

  # calculate retained earnings
  $form->{earnings_this_period} = $form->{total_assets_this_period} - $form->{total_liabilities_this_period} - $form->{total_equity_this_period};

  push(@{$form->{equity_this_period}}, $form->format_amount($myconfig, $form->{earnings_this_period}, 1));
  
  $form->{total_equity_this_period} = sprintf("%.2f", $form->{total_equity_this_period} + $form->{earnings_this_period});
  
  # add liability + equity
  $form->{total_this_period} = $form->format_amount($myconfig, $form->{total_liabilities_this_period} + $form->{total_equity_this_period}, 1);


  if ($last_period) {
    # totals for assets, liabilities
    $form->{total_assets_last_period} = sprintf("%.2f", $form->{total_assets_last_period});
    $form->{total_liabilities_last_period} = sprintf("%.2f", $form->{total_liabilities_last_period});
    

    # calculate retained earnings
    $form->{earnings_last_period} = $form->{total_assets_last_period} - $form->{total_liabilities_last_period} - $form->{total_equity_last_period};

    push(@{$form->{equity_last_period}}, $form->format_amount($myconfig,$form->{earnings_last_period}, 1));
    
    $form->{total_equity_last_period} = sprintf("%.2f", $form->{total_equity_last_period} + $form->{earnings_last_period});

    # add liability + equity
    $form->{total_last_period} = $form->format_amount($myconfig, $form->{total_liabilities_last_period} + $form->{total_equity_last_period}, 1);

  }

  
  $form->{total_liabilities_last_period} = $form->format_amount($myconfig, $form->{total_liabilities_last_period}, 1) if ($form->{total_liabilities_last_period} != 0);
  
  $form->{total_equity_last_period} = $form->format_amount($myconfig, $form->{total_equity_last_period}, 1) if ($form->{total_equity_last_period} != 0);
  
  $form->{total_assets_last_period} = $form->format_amount($myconfig, $form->{total_assets_last_period}, 1) if ($form->{total_assets_last_period} != 0);
  
  $form->{total_assets_this_period} = $form->format_amount($myconfig, $form->{total_assets_this_period}, 1);
  
  $form->{total_liabilities_this_period} = $form->format_amount($myconfig, $form->{total_liabilities_this_period}, 1);
  
  $form->{total_equity_this_period} = $form->format_amount($myconfig, $form->{total_equity_this_period}, 1);
  
}


sub get_accounts {
  my ($dbh, $last_period, $where, $category, $form) = @_;

  my $query = qq|SELECT chart.accno, sum(acc_trans.amount) AS amount, 
                 chart.description
                 FROM acc_trans, chart
                 WHERE $where
	         AND chart.id = acc_trans.chart_id
 	         AND chart.category = '$category'
		 GROUP BY accno, description
	         |;
  my $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);

  while (my $ref = $sth->fetchrow_hashref) {
    next if (sprintf("%.2f", $ref->{amount}) == 0);

    $form->{$category}{$ref->{accno}}{description} = $ref->{description};
    if ($last_period) {
      $form->{$category}{$ref->{accno}}{last} = $ref->{amount};
    } else {
      $form->{$category}{$ref->{accno}}{this} = $ref->{amount};
    }
  }

  $sth->finish;

}



# Antonio Gallardo
# for each account that have transactions on the period
# the sub get his Initial Balance (IB) and the sum of
# credits and debits for the period.
sub trial_balance_details {
  my ($self, $myconfig, $form) = @_;

  my $dbh = $form->dbconnect($myconfig);

  # if there are any dates construct a where
  my $where = "true";
  
  if ($form->{fromdate} || $form->{todate}) {
    if ($form->{fromdate}) {
      $where .= " AND transdate >= '$form->{fromdate}'";
    }
    if ($form->{todate}) {
      $where .= " AND transdate <= '$form->{todate}'";
    }
  }

  # First we take the sum of debit and credit of each account in the period
  $query = qq|SELECT accno, description, charttype, category,
              SUM(amount) AS amount
              FROM chart, acc_trans
              WHERE chart.id = acc_trans.chart_id
	      AND $where
              GROUP BY accno, description, charttype, category
           UNION
	      SELECT accno, description, charttype, category, 0.0 AS amount
	      FROM chart
	      WHERE charttype = 'H'
	      ORDER BY accno
             |;
  $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);

  # calculate the debit and credit in the period
  while (my $ref = $sth->fetchrow_hashref) {
    if ($ref->{amount} < 0) {
      $ref->{debit} = -$ref->{amount};
    } else {
      $ref->{credit} = $ref->{amount};
    }
    push @{ $form->{TB} }, $ref;
  }

  $sth->finish;
  $dbh->disconnect;
  
}


sub aging {
  my ($self, $myconfig, $form) = @_;

  # connect to database
  my $dbh = $form->dbconnect($myconfig);

  $form->{todate} = $form->current_date($myconfig) unless ($form->{todate});

  my $where = ($form->{"$form->{ct}_id"}) ? qq|$form->{ct}.id = $form->{"$form->{ct}_id"}| : TRUE;

  # select outstanding vendors or customers, depends on $ct
  my $query = qq|SELECT DISTINCT $form->{ct}.id, name
                 FROM $form->{ct}, $form->{arap}
		 WHERE $where
                 AND $form->{arap}.$form->{ct} = $form->{ct}.id
                 AND paid != amount
                 AND (transdate <= date '$form->{todate}')
                 ORDER BY name|;
  my $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror;


  # for each company that has some stuff outstanding
  while ( my ($outstanding_entity_id) = $sth->fetchrow_array ) {

    $query = qq|

-- between 0-30 days

	SELECT $form->{ct}.id, $form->{ct}.name,
	"invnumber", "transdate",
	(amount - paid) as "c0", 0.00 as "c30", 0.00 as "c60", 0.00 as "c90",
	duedate
  FROM $form->{arap}, $form->{ct} 
	WHERE paid != amount
	AND $form->{arap}.$form->{ct} = $form->{ct}.id
	AND $form->{ct}.id = $outstanding_entity_id
	AND (
	        transdate <= (date '$form->{todate}' - interval '0 days') 
	        AND transdate >= (date '$form->{todate}' - interval '30 days')
	    )
	
	UNION

-- between 31-60 days

	SELECT $form->{ct}.id, $form->{ct}.name,
	"invnumber", "transdate", 
	0.00 as "c0", (amount - paid) as "c30", 0.00 as "c60", 0.00 as "c90",
	duedate
  FROM $form->{arap}, $form->{ct}
	WHERE paid != amount 
	AND $form->{arap}.$form->{ct} = $form->{ct}.id 
	AND $form->{ct}.id = $outstanding_entity_id
	AND (
		transdate < (date '$form->{todate}' - interval '30 days') 
		AND transdate >= (date '$form->{todate}' - interval '60 days')
		)

	UNION
  
-- between 61-90 days

	SELECT $form->{ct}.id, $form->{ct}.name,
	"invnumber", "transdate", 
	0.00 as "c0", 0.00 as "c30", (amount - paid) as "c60", 0.00 as "c90",
	duedate
	FROM $form->{arap}, $form->{ct} 
	WHERE paid != amount
	AND $form->{arap}.$form->{ct} = $form->{ct}.id 
	AND $form->{ct}.id = $outstanding_entity_id
	AND (
		transdate < (date '$form->{todate}' - interval '60 days') 
		AND transdate >= (date '$form->{todate}' - interval '90 days')
		)

	UNION
  
-- over 90 days

	SELECT $form->{ct}.id, $form->{ct}.name,
	"invnumber", "transdate", 
	0.00 as "c0", 0.00 as "c30", 0.00 as "c60", (amount - paid) as "c90",
	duedate
	FROM $form->{arap}, $form->{ct} 
	WHERE paid != amount
	AND $form->{arap}.$form->{ct} = $form->{ct}.id 
	AND $form->{ct}.id = $outstanding_entity_id
	AND transdate < (date '$form->{todate}' - interval '90 days') 

	ORDER BY
  
  id, invnumber, transdate
  
		|;

    my $sth = $dbh->prepare($query);
    $sth->execute || $form->dberror;

    while (my $ref = $sth->fetchrow_hashref) {
      push @{ $form->{AG} }, $ref;
    }
    
    $sth->finish;

  }

  $sth->finish;
  # disconnect
  $dbh->disconnect;

}


1;


