#=====================================================================
# SQL-Ledger Accounting
# Copyright (C) 2001
#
#  Author: Dieter Simader
#   Email: dsimader@sql-ledger.org
#     Web: http://www.sql-ledger.org
#
#  Contributors: Martin Lillepuu <martin@lillepuu.com>
#                Thomas Good <tomg@admin.nrnet.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#=====================================================================
#
# user related functions
#
# CHANGE LOG:
#   DS. 2000-07-04  Created
#   ML. 2001-02-20  changed crypt to use first two characters of login
#   TG. 2001-08-01  Oracle code
#
#=====================================================================

package User;


sub new {
  my ($type, $memfile, $login) = @_;
  my $self = {};

  if ($login ne "") {
    open(MEMBER, "$memfile") || &error("$!: $memfile");
    while (<MEMBER>) {
      if (/^\[$login\]/) {
	while (<MEMBER>) {
	  last if /^\[/;
	  next if /^(#|\s)/;
	  
	  # remove comments
	  s/\s#.*//g;

	  # remove any trailing whitespace
	  s/^\s*(.*?)\s*$/$1/;

	  ($key, $value) = split /=/, $_, 2;
	  
	  $self->{$key} = $value;
	}
	
	$self->{login} = $login;

	last;
      }
    }
    close(MEMBER);
  }

  bless $self, $type;
}

 

sub login {
  my ($self, $form, $userspath) = @_;

  
  if ($form->{login} eq "") {
    $form->error(qq|You did not enter a name|);
    return;
  }


  if ($self->{login}) {
    
    if ($self->{password}) {
      if ($self->{password} ne crypt $form->{password}, substr($self->{login}, 0, 2)) {
	$form->error(qq|Incorrect Password|);
	return;
      }
    }
    
    unless (-e "$userspath/$form->{login}.conf") {
      $self->create_config("$userspath/$form->{login}.conf");
    }
    
    if ($self->dbdown($form, $userspath)) {
      $form->error(qq|Database is offline; please try again later|);
      return;
    }
  } else {
    $form->error(qq|$form->{login} is not a member|);
    return;
  }

  1;

}



sub dbdown {
  my ($self, $form, $userspath) = @_;

  require("$userspath/$form->{login}.conf");
  $myconfig{dbpasswd} = unpack 'u', $myconfig{dbpasswd};
  
  # check if database is down
  my $dbh = DBI->connect($myconfig{dbconnect}, $myconfig{dbuser}, $myconfig{dbpasswd});
  
  $errstr = DBI::errstr;

  if ($errstr =~ /connect.*failed/i) {
    return 1;
  }

  $dbh->disconnect;
  return 0;
  
}


sub dbconnect_vars {
  my ($form, $db) = @_;
  
  my %dboptions = (
     'Pg' => {
        'yy-mm-dd' => 'set DateStyle to ISO',
	'mm/dd/yy' => 'set DateStyle to SQL, US',
	'mm-dd-yy' => 'set DateStyle to POSTGRES, US',
	'dd/mm/yy' => 'set DateStyle to SQL, EUROPEAN',
	'dd-mm-yy' => 'set DateStyle to POSTGRES, EUROPEAN',
	'dd.mm.yy' => 'set DateStyle to GERMAN'
	     },
     'Oracle' => {
	'yy-mm-dd' => 'ALTER SESSION SET NLS_DATE_FORMAT = \'YY-MM-DD\'',
	'mm/dd/yy' => 'ALTER SESSION SET NLS_DATE_FORMAT = \'MM/DD/YY\'',
	'mm-dd-yy' => 'ALTER SESSION SET NLS_DATE_FORMAT = \'MM-DD-YY\'',
	'dd/mm/yy' => 'ALTER SESSION SET NLS_DATE_FORMAT = \'DD/MM/YY\'',
	'dd-mm-yy' => 'ALTER SESSION SET NLS_DATE_FORMAT = \'DD-MM-YY\'',
	'dd.mm.yy' => 'ALTER SESSION SET NLS_DATE_FORMAT = \'DD.MM.YY\'',
	         }
     );
			     
  $form->{dboptions} = $dboptions{$form->{dbdriver}}{$form->{dateformat}};

  if ($form->{dbdriver} eq 'Pg') {
    $form->{dbconnect} = "dbi:Pg:dbname=$db"
  }

  if ($form->{dbdriver} eq 'Oracle') {
    $form->{dbconnect} = "dbi:Oracle:$db;sid=$form->{sid}";
  }

  if ($form->{dbhost}) {
    $form->{dbconnect} .= ";host=$form->{dbhost}";
  }
  if ($form->{dbport}) {
    $form->{dbconnect} .= ";port=$form->{dbport}";
  }
  
}


sub dbdrivers {

  my @drivers = DBI->available_drivers();

  return (grep { /Pg/ } @drivers);
#  return (grep { /(Pg|Oracle)/ } @drivers);

}


sub dbsources {
  my ($self, $form) = @_;

  my @dbsources = ();
  
  my %syscat = ( 'Pg' => { 'datname' => 'datname',
                         'sysdb' => 'pg_database',
		 	 'tablename' => 'tablename',
			 'systables' => 'pg_tables' },
             'Oracle' => { 'datname' => 'tablespace_name',
		         'sysdb' => 'dba_tablespaces',
		 	 'tablename' => 'obj_name',
			 'systables' => 'dba_objects' }
               );

  &dbconnect_vars($form, $form->{dbdefault});

  my $dbh = DBI->connect($form->{dbconnect}, $form->{dbuser}, $form->{dbpasswd}) || $form->dberror;
  my $query = qq|SELECT $syscat{$form->{dbdriver}}{datname} FROM $syscat{$form->{dbdriver}}{sysdb}|;
  my $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);

  while (my ($db) = $sth->fetchrow_array) {
    if ($form->{only_acc_db}) {
      
      &dbconnect_vars($form, $db);
      my $dbh = DBI->connect($form->{dbconnect}, $form->{dbuser}, $form->{dbpasswd}) || $form->dberror;

      $query = qq|SELECT $syscat{$form->{dbdriver}}{tablename} FROM $syscat{$form->{dbdriver}}{systables}
	 	  WHERE $syscat{$form->{dbdriver}}{tablename} = 'defaults'|;
      my $sth = $dbh->prepare($query);
      $sth->execute || $form->dberror($query);

      if ($sth->fetchrow_array) {
	push @dbsources, $db;
      }
      $sth->finish;
      $dbh->disconnect;
      next;
    }
    
    push @dbsources, $db;
    
  }

  $sth->finish;

  $dbh->disconnect;
  
  return @dbsources;

}


sub dbusers {
  my ($self, $form) = @_;

  my @dbusers = ();
  
  my %syscat = ( 'Pg' => { 'usename' => 'usename',
                         'tablename' => 'pg_user' },
             'Oracle' => { 'usename' => 'username',
	                 'tablename' => 'dba_users' }
               );

  &dbconnect_vars($form, $form->{dbdefault});
  my $dbh = DBI->connect($form->{dbconnect}, $form->{dbuser}, $form->{dbpasswd}) || $form->dberror;
  my $query = qq|SELECT $syscat{$form->{dbdriver}}{usename} FROM $syscat{$form->{dbdriver}}{tablename}|;
  my $sth = $dbh->prepare($query);
  $sth->execute || $form->dberror($query);

  while (my ($name) = $sth->fetchrow_array) {
    push @dbusers, $name;
  }
  $sth->finish;
  $dbh->disconnect;

  return @dbusers;

}


sub dbcreate {
  my ($self, $form) = @_;

  my %dbcreate = ( 'Pg' => "CREATE DATABASE $form->{db}",
               'Oracle' => "CREATE TABLESPACE $form->{db} DATAFILE '$form->{db}.dbf'");

  &dbconnect_vars($form, $form->{dbdefault});
  my $dbh = DBI->connect($form->{dbconnect}, $form->{dbuser}, $form->{dbpasswd}) || $form->dberror;
  my $query = qq|$dbcreate{$form->{dbdriver}}|;
  $dbh->do($query) || $form->dberror($query);

  $dbh->disconnect;


  # connect to the new database
  &dbconnect_vars($form, $form->{db});
  $dbh = DBI->connect($form->{dbconnect}, $form->{dbuser}, $form->{dbpasswd}) || $form->dberror;
  # create the tables
  my $filename = qq|$form->{dbdriver}-tables.sql|;
  open FH, "sql/$filename" || $form->error("Can't open sql/$filename: $!\n");
  $query = "";
  while (<FH>) {
    next if (/^(--|\s*$)/);
    $query .= $_;

    if (/;\s*$/) {
      $dbh->do($query) || $form->dberror($query);
      $query = "";
    }
  }
  close FH;

 
  # load a chart of accounts
  $filename = qq|$form->{chart}-chart.sql|;
  open FH, "sql/$filename" || $form->error("Can't open sql/$filename: $!\n");
  $query = "";
  while (<FH>) {
    next if (/^(--|\s*$)/);
    $query .= $_;

    if (/;\s*$/) {
      $dbh->do($query) || $form->dberror($query);
      $query = "";
    }
  }
  close FH;

  $dbh->disconnect;

}
  

sub dbdelete {
  my ($self, $form) = @_;

  my %dbdelete = ( 'Pg' => "DROP DATABASE $form->{db}",
               'Oracle' => "DROP TABLESPACE $form->{db} INCLUDING CONTENTS"
	         );
  
  &dbconnect_vars($form, $form->{dbdefault});
  my $dbh = DBI->connect($form->{dbconnect}, $form->{dbuser}, $form->{dbpasswd}) || $form->dberror;
  my $query = qq|$dbdelete{$form->{dbdriver}}|;
  $dbh->do($query) || $form->dberror($query);

  $dbh->disconnect;

}
  


sub dbsources_unused {
  my ($self, $form, $memfile) = @_;

  my @dbexcl = ();
  my @dbsources = ();
  
  # open members file
  open FH, "$memfile" || $form->error("Can't open $memfile: $!");

  while (<FH>) {
    if (/^dbname=/) {
      my ($null,$item) = split /=/;
      push @dbexcl, $item;
    }
  }

  close FH;
  

  $form->{only_acc_db} = 1;
  my @db = &dbsources("", $form);

  push @dbexcl, $form->{dbdefault};

  foreach $item (@db) {
    unless (grep /$item/, @dbexcl) {
      push @dbsources, $item;
    }
  }

  return @dbsources;

}



sub create_config {
  my ($self, $filename) = @_;


  @config = &config_vars;
  
  open CONF, ">$filename" || $self->error("$!: $filename");
  
  # create the config file
  print CONF qq|# configuration file for $self->{login}

\%myconfig = (
|;

  foreach $key (sort @config) {
    $self->{$key} =~ s/'/\\'/g;
    print CONF qq|  $key => '$self->{$key}',\n|;
  }

   
  print CONF qq|);\n\n|;

  close CONF;

}


sub save_member {
  my ($self, $memberfile, $userspath) = @_;

  # format dbconnect and dboptions string
  &dbconnect_vars($self, $self->{dbname});
  
  # no error trapping, if the file doesn't exist create it!
  open(CONF, $memberfile) || $self->error("$!: $memberfile");
  @config = <CONF>;
  close CONF;
  
  open(CONF, ">$memberfile") || $self->error("$!: $memberfile");

  while ($line = shift @config) {
    last if ($line =~ /^\[$self->{login}\]/);
    print CONF $line;
  }

  # remove everything up to next login or EOF
  while ($line = shift @config) {
    last if ($line =~ /^\[/);
  }

  # this one is either the next login or EOF
  print CONF $line;

  while ($line = shift @config) {
    print CONF $line;
  }

  print CONF qq|[$self->{login}]\n|;

  # encrypt passwords
  # can't use crypt for the dbpassword because we need to send it
  # in plain text to the server, so we uuencode it instead
  if ($self->{dbpasswd} && $self->{root}) {
    $self->{dbpasswd} = pack 'u', $self->{dbpasswd};
    chop $self->{dbpasswd};
  }
  
  if (($self->{password} ne $self->{old_password}) && $self->{password}) {
    $self->{password} = crypt $self->{password}, substr($self->{login}, 0, 2);
  }
  
  ($self->{countrycode}) = split /--/, $self->{countrycode};
  
  if ($self->{'root login'}) {
    @config = ("password");
  } else {
    @config = &config_vars;
  }
  
  # replace ^M$ with \n
  $self->{address} =~ s/
\n/\\n/g if $self->{address};
  $self->{signature} =~ s/
\n/\\n/g if $self->{signature};

  foreach $key (sort @config) {
    print CONF qq|$key=$self->{$key}\n|;
  }

  print CONF "\n";
  close(CONF);

  # create conf file
  unless ($self->{'root login'}) {
    $self->create_config("$userspath/$self->{login}.conf");
  }

}


sub config_vars {
  
  my @conf = qw(acs address admin businessnumber charset company countrycode
             dateformat dbconnect dbdriver dbhost dbname dboptions dbpasswd
	     dbport dbtty dbuser email fax name numberformat password printer
	     shippingpoint signature stylesheet tel templates);

  @conf;

}


sub error {
  my ($msg) = @_;

  if ($ENV{HTTP_USER_AGENT}) {
    print qq|Content-Type: text/html

<body bgcolor=ffffff>

<h2><font color=red>Error!</font></h2>
<p><b>$msg</b>|;

  }
  
  die "Error: $msg\n";
  
}


1;

