#=====================================================================
# SQL-Ledger Accounting
# Copyright (c) 2001
#
#  Author: Dieter Simader
#   Email: dsimader@sql-ledger.org
#     Web: http://www.sql-ledger.org
#
#  Contributors:
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#======================================================================
#
# Accounts Receivables
#
# CHANGE LOG:
#   DS. 2000-07-04  Created
#   DS. 2001-07-25  added number formatting
#   DS. 2001-08-19  due date and tax calculation
#
#======================================================================


use DBI;
use SL::Form;
use SL::AR;
use SL::IS;


$form = new Form;


if ($form->{action}) {
  require("$userspath/$form->{login}.conf");
  $myconfig{dbpasswd} = unpack 'u', $myconfig{dbpasswd};
  map { $form->{$_} = $myconfig{$_} } qw(stylesheet charset);

  $locale = new Locale $myconfig{countrycode}, "ar";
  &{ $locale->findsub($form->{action}) };
} else {
  $form->error("action= not defined!");
}

# end of main

# this is for our long dates
# $locale->text('January')
# $locale->text('February')
# $locale->text('March')
# $locale->text('April')
# $locale->text('May')
# $locale->text('June')
# $locale->text('July')
# $locale->text('August')
# $locale->text('September')
# $locale->text('October')
# $locale->text('November')
# $locale->text('December')

# this is for our short month
# $locale->text('Jan')
# $locale->text('Feb')
# $locale->text('Mar')
# $locale->text('Apr')
# $locale->text('May')
# $locale->text('Jun')
# $locale->text('Jul')
# $locale->text('Aug')
# $locale->text('Sep')
# $locale->text('Oct')
# $locale->text('Nov')
# $locale->text('Dec')


sub add {

  $form->{title} = "Add";
  $form->{callback} = "$form->{script}?action=add&path=$form->{path}&login=$form->{login}";

  &create_links;
  &display_form;
  
}


sub edit {

  $form->{title} = "Edit";

  &create_links;
  &display_form;

}


sub display_form {

  &form_header;
  &form_footer;

}


sub create_links {
  
  $form->create_links("AR", \%myconfig, "customer");

  $taxincluded = $form->{taxincluded};
  
  # build the popup menus
  foreach $ref (@{ $form->{all_customer} }) {
    if ($ref->{id} == $form->{customer_id}) {
      $form->{customer} = "$ref->{name}--$ref->{id}";
      $form->{customer_id} = $ref->{id};
      IS->get_customer(\%myconfig, \%$form);
    }
    $form->{selectcustomer} .= "<option>$ref->{name}--$ref->{id}\n";
  }
  

  foreach $key (keys %{ $form->{AR_links} }) {
    foreach $ref (@{ $form->{AR_links}{$key} }) {
      
      $nkey = $key;
      
      if ($key eq "AR_tax") {
	$nkey = "${key}_$ref->{accno}";
      }
      
      if ($form->{amount}{$ref->{accno}}) {
        # if there is a value we have an old entry
	$form->{$nkey} = "$ref->{accno}--$ref->{description}";
	$form->{amount}{$nkey} = $form->{amount}{$ref->{accno}};
      }
      $form->{"select$nkey"} .= "<option>$ref->{accno}--$ref->{description}\n";
    }
  }

  $form->{taxincluded} = $taxincluded if ($form->{id});
  
  # reverse paid
  $form->{paid} = $form->{amount}{AR_paid} * -1;

  #taxes
  foreach $item (split / /, $form->{taxaccounts}) {
    $form->{"tax_$item"} = $form->{amount}{"AR_tax_$item"};
    $totaltax += $form->{"tax_$item"};
  }

  $form->{amount}{AR_amount} += $totaltax if ($form->{taxincluded});
  
  # amount
  $form->{amount} = $form->{amount}{AR_amount};

}


sub form_header {

  $form->{title} = $locale->text("$form->{title} Accounts Receivables");

  $form->{taxincluded} = ($form->{taxincluded}) ? "checked" : "";
  
# this is for our parser only!
# class=submit type=submit $locale->text('Add Accounts Receivables')
# class=submit type=submit $locale->text('Edit Accounts Receivables')
 
  # set option selected
  map { $form->{$_} =~ s/ selected// } qw(selectAR selectAR_amount selectAR_paid selectcustomer);
  map { $form->{"select${_}"} =~ s/option>$form->{$_}/option selected>$form->{$_}/ } qw(AR AR_amount AR_paid customer);

  # format amounts
  map { $form->{$_} = $form->format_amount(\%myconfig, $form->{$_}) } qw(amount paid);


  $form->header;
  
  print qq|

<h2>$form->{title}</h2>

<form method=post action=$form->{script}>

<input type=hidden name=id value=$form->{id}>
<input type=hidden name=sort value=$form->{sort}>


<table border=1>
<tr valign=top>
<td>

<table>

<tr>
  <th align=right>|.$locale->text('Record in').qq|</th>
  <td colspan=3><select name=AR>$form->{selectAR}</select></td>
  <input type=hidden name=selectAR value="$form->{selectAR}">
</tr>

<tr>
  <th align=right>|.$locale->text('Customer').qq|</th>
  <td colspan=3><select name=customer>$form->{selectcustomer}</select></td>
  <input type=hidden name=selectcustomer value="$form->{selectcustomer}">
  <input type=hidden name=oldcustomer value="$form->{customer_id}">
  <input type=hidden name=terms value=$form->{terms}>
</tr>

<tr>
  <th align=right>|.$locale->text('Tax Included').qq|</th>
  <td colspan=3><input name=taxincluded class=checkbox type=checkbox value=1 $form->{taxincluded}></td>
</tr>

<tr>
  <th align=right>|.$locale->text('Invoice Number').qq|</th>
  <td colspan=3><input name=invnumber size=20 value="$form->{invnumber}"></td>
</tr>

<tr>
  <th align=right>|.$locale->text('Invoice Date').qq|<br>($myconfig{'dateformat'})</th>
  <td><input name=transdate size=10 value=$form->{transdate}></td>
  <th align=right>|.$locale->text('Due Date').qq|<br>($myconfig{'dateformat'})</th>
  <td><input name=duedate size=10 value=$form->{duedate}></td>
</tr>

<tr>
  <th align=right>|.$locale->text('Amount').qq|</th>
  <td><input name=amount size=10 value=$form->{amount}></td>
  <td colspan=2><select name=AR_amount>$form->{selectAR_amount}</select></td>
  <input type=hidden name=selectAR_amount value="$form->{selectAR_amount}">
</tr>

<input type=hidden name=taxaccounts value="$form->{taxaccounts}">
|;

  
  $taxlabel = ($form->{taxincluded}) ? $locale->text('Tax Included') : $locale->text('Tax');

  
  foreach $item (split / /, $form->{taxaccounts}) {

    $form->{"tax_$item"} = $form->format_amount(\%myconfig, $form->{"tax_$item"});

    print qq|
<tr>
  <th align=right>$taxlabel</th>
  <td><input name="tax_$item" size=9 value=$form->{"tax_$item"}></td>
  <td colspan=2><select name="AR_tax_$item">$form->{"selectAR_tax_$item"}</select></td>
</tr>

<input type=hidden name="${item}_rate" value="$form->{"${item}_rate"}">
<input type=hidden name="${item}_description" value="$form->{"${item}_description"}">
<input type=hidden name="selectAR_tax_$item" value="$form->{"selectAR_tax_$item"}">
|;
  
  }

  print qq|
<tr>
  <th align=right>|.$locale->text('Paid').qq|</th>
  <td><input name=paid size=10 value=$form->{paid}></td>
  <td colspan=2><select name=AR_paid>$form->{selectAR_paid}</select></td>
  <input type=hidden name=selectAR_paid value="$form->{selectAR_paid}">
</tr>

<tr>
  <th align=right>|.$locale->text('Date Paid').qq| ($myconfig{dateformat})</th>
  <td><input name=datepaid size=10 value=$form->{datepaid}></td>
</tr>
</table>

</td>
</tr>

|;

}


sub form_footer {

  print qq|

<input name=callback type=hidden value="$form->{callback}">

<input type=hidden name=path value=$form->{path}>
<input type=hidden name=login value=$form->{login}>

</table>

<br>
<input class=submit type=submit name=action value="|.$locale->text('Update Form').qq|">
<input class=submit type=submit name=action value="|.$locale->text('Post Transaction').qq|">

|;

  if ($form->{id}) {
    print qq|<input class=submit type=submit name=action value="|.$locale->text('Delete Transaction').qq|">|;
  }

  print "</form>";

}


sub update_form {

  # split customer
  ($null, $form->{customer_id}) = split /--/, $form->{customer};
  
  # recalculate form
  if ($form->{oldcustomer} != $form->{customer_id}) {
    IS->get_customer(\%myconfig, \%$form);
  }

  # recalculate due date
  $form->{duedate} = $form->current_date(\%myconfig, $form->{transdate}, $form->{terms});

  $form->{amount} = $form->parse_amount(\%myconfig, $form->{amount});

  # recalculate taxes
  if ($form->{taxincluded}) {
    $taxrate = 0;

    map { $taxrate += $form->{"${_}_rate"} } split / /, $form->{taxaccounts};

    foreach $item (split / /, $form->{taxaccounts}) {
      $form->{"tax_$item"} = sprintf("%.2f", $form->{amount} * (1 - 1 / (1 + $taxrate)) * $form->{"${item}_rate"} / $taxrate);
      $form->{"selectAR_tax_$item"} = qq|<option>$item--$form->{"${item}_description"}|;
      $totaltax += $form->{"tax_$item"};
    }
  } else {
    foreach $item (split / /, $form->{taxaccounts}) {
      $form->{"tax_$item"} = $form->{amount} * $form->{"${item}_rate"};
      $form->{"selectAR_tax_$item"} = qq|<option>$item--$form->{"${item}_description"}|;
      $totaltax += $form->{"tax_$item"};
    }
  }

  $form->{paid} = ($form->{taxincluded}) ? $form->{amount} : $form->{amount} + $totaltax;
  $form->{datepaid} = $form->{transdate};
  
  &display_form;
  
}


sub post_transaction {

  # check if there is an invoice number, invoice and due date
  $form->isblank("invnumber", $locale->text('Invoice Number missing!'));
  $form->isblank("transdate", $locale->text('Invoice Date missing!'));
  $form->isblank("duedate", $locale->text('Due Date missing!'));
  
  # split customer
  ($null, $form->{customer_id}) = split /--/, $form->{customer};
  
  # recalculate form
  if ($form->{oldcustomer} != $form->{customer_id}) {
    IS->get_customer(\%myconfig, \%$form);
  }

  $form->isposted(AR->post_transaction(\%myconfig, \%$form));

}



sub delete_transaction {

  $form->{title} = $locale->text('Confirm!');
  
  $form->header;

  print qq|<form method=post action=$form->{script}>\n|;

  foreach $key (keys %$form) {
    print qq|<input type=hidden name=$key value="$form->{$key}">\n|;
  }

  print qq|
<h2><font color=red>$form->{title}</font></h2>

<h4>|.$locale->text('Are you sure you want to delete Transaction').qq| $form->{invnumber}</h4>
<p>

<input name=action class=submit type=submit value="|.$locale->text('Yes Delete Transaction').qq|">
</form>
|;

}



sub yes_delete_transaction {

  $form->isdeleted(AR->delete_transaction(\%myconfig, \%$form));

}


sub search {
  
  # setup customer selection
  $form->all_vc(\%myconfig, "customer");
  
  $selection = "<option selected>" . $locale->text('All Customers') . "\n";
  
  foreach $ref (@{ $form->{all_customer} }) {
    $selection .= "<option>$ref->{name}--$ref->{id}\n";
  }
  

  if ($form->{nextsub} eq "receipts") {
    $form->{title} = $locale->text('Process Receipts');
    $listopen = qq|
    <input type=hidden name=sort value=invnumber>
    <input type=hidden name=open value=Y>|;
  }
  if ($form->{nextsub} eq "ar_transactions") {
    $form->{title} = $locale->text('AR Transactions');
    $listopen = qq|
<input type=hidden name=sort value=transdate>
<tr>
<th align=right>|.$locale->text('List open transaction').qq|</th><td><input name=open class=checkbox type=checkbox value=Y></td>
</tr>
<tr>
<th align=right>|.$locale->text('Include in Report').qq|</th>
<td colspan=3>
<input name="l_invnumber" class=checkbox type=checkbox value=Y checked>&nbsp;|.$locale->text('Invoice Number').qq|
<input name="l_transdate" class=checkbox type=checkbox value=Y checked>&nbsp;|.$locale->text('Invoice Date').qq|
<input name="l_duedate" class=checkbox type=checkbox value=Y checked>&nbsp;|.$locale->text('Due Date').qq|
 <input name="l_name" class=checkbox type=checkbox value=Y checked>&nbsp;|.$locale->text('Customer').qq|
 <input name="l_netamount" class=checkbox type=checkbox value=Y>&nbsp;|.$locale->text('Net Amount').qq|
 <input name="l_tax" class=checkbox type=checkbox value=Y>&nbsp;|.$locale->text('Tax').qq|
 <input name="l_amount" class=checkbox type=checkbox value=Y checked>&nbsp;|.$locale->text('Total').qq|
<input name="l_paid" class=checkbox type=checkbox value=Y checked>&nbsp;|.$locale->text('Paid') .qq|
<input name="l_datepaid" class=checkbox type=checkbox value=Y>&nbsp;|.$locale->text('Date Paid').qq|
<input name="l_terms" class=checkbox type=checkbox value=Y>&nbsp;|.$locale->text('Terms').qq|
<br>
<input name="l_subtotal" class=checkbox type=checkbox value=Y>&nbsp;|.$locale->text('Subtotal').qq|
</td></tr>
|;

  }
    
  $form->header;
  
  print qq|<h2>$form->{title}</h2>

<form method=post action=$form->{script}>

<table border=1>
<tr><td>

<table>

<tr>
<th align=right>|.$locale->text('Customer').qq|</th><td colspan=3><select name=customer>$selection</select></td>
</tr>

<tr>
<th align=right>|.$locale->text('Invoice Number').qq|</th><td><input name=invnumber size=20></td>
</tr>

<tr>
<th align=right>|.$locale->text('From').qq| ($myconfig{dateformat})</th><td><input name=transdatefrom size=10></td>
<th align=right>|.$locale->text('To').qq|</th><td><input name=transdateto size=10></td>
</tr>

$listopen


</table>

</td></tr>
</table>

<p>
<input type=hidden name=nextsub value=$form->{nextsub}>

<input type=hidden name=path value=$form->{path}>
<input type=hidden name=login value=$form->{login}>

<input class=submit type=submit name=action value="|.$locale->text('Search for Transaction').qq|">
</form>

<p>|.$locale->text('To display all transactions, leave fields blank');

}


sub search_for_transaction {

  &{ $form->{nextsub} };

}


sub ar_transactions {

  # split customer
  ($customer, $form->{customer_id}) = split(/--/, $form->{customer});
  
  AR->ar_transactions(\%myconfig, \%$form);

  # construct href
  $href = "$form->{script}?path=$form->{path}&action=ar_transactions&invnumber=" . $form->escape($form->{invnumber}) . "&transdatefrom=$from->{transdatefrom}&transdateto=$form->{transdateto}&open=$form->{open}&customer=" . $form->escape($form->{customer}) . "&login=$form->{login}";
  
  # construct callback
  $customer = $form->{customer};
  $customer =~ s/&/_/g;
  $callback = "$form->{script}?action=ar_transactions&transdatefrom=$from->{transdatefrom}&transdateto=$form->{transdateto}&open=$form->{open}&login=$form->{login}&path=$form->{path}&invnumber=$form->{invnumber}&customer=$customer";

  @columns = $form->sort_columns(qw(transdate duedate invnumber name netamount tax amount paid datepaid terms));
  
  foreach $item (@columns) {
    if ($form->{"l_$item"} eq "Y") {
      push @column_index, $item;

      # add column to href and callback
      $callback .= "&l_$item=Y";
      $href .= "&l_$item=Y";
    }
  }

  if ($form->{l_subtotal} eq 'Y') {
    $callback .= "&l_subtotal=Y";
    $href .= "&l_subtotal=Y";
  }
  

  $column_header{transdate} = "<th><a href=$href&sort=transdate><font color=ffffff>".$locale->text('Date')."</a></th>";
  $column_header{duedate} = "<th><a href=$href&sort=duedate><font color=ffffff>".$locale->text('Due')."</a></th>";
  $column_header{invnumber} = "<th><a href=$href&sort=invnumber><font color=ffffff>".$locale->text('Invoice')."</a></th>";
  $column_header{name} = "<th><a href=$href&sort=name><font color=ffffff>".$locale->text('Customer')."</a></th>";
  $column_header{netamount} = "<th><font color=ffffff>" . $locale->text('Net Amount') . "</th>";
  $column_header{tax} = "<th><font color=ffffff>" . $locale->text('Tax') . "</th>";
  $column_header{amount} = "<th><font color=ffffff>" . $locale->text('Amount') . "</th>";
  $column_header{paid} = "<th><font color=ffffff>" . $locale->text('Paid') . "</th>";
  $column_header{datepaid} = "<th><a href=$href&sort=datepaid><font color=ffffff>" . $locale->text('Date Paid') . "</a></th>";
  $column_header{terms} = "<th><font color=ffffff>" . $locale->text('Terms') . "</th>";


  $form->{title} = $locale->text('AR Transactions');
  
  ($customer) = split /--/, $form->{customer};
  if ($form->{customer_id}) {
    $option .= $customer;
  }
  if ($form->{transdatefrom}) {
    $option .= "<br>".$locale->text('From').": ".$locale->date(\%myconfig, $form->{transdatefrom}, 1);
  }
  if ($form->{transdateto}) {
    $option .= "<br>".$locale->text('To').": ".$locale->date(\%myconfig, $form->{transdateto}, 1);
  }
  
  $form->header;

  print qq|
<h2>$form->{title}</h2>

<b>$option</b>

<p>
<table border=1>
<tr bgcolor=336666>
|;

  map { print "$column_header{$_}\n" } @column_index;

  print qq|
</tr>
|;


  # add sort and escape callback
  $callback = $form->escape($callback . "&sort=$form->{sort}");

  $sameitem = $form->{AR}->[0]->{$form->{sort}};

  # sums and tax on reports by Antonio Gallardo
  #
  foreach $ar (@{ $form->{AR} }) {
  
    if ($form->{l_subtotal} eq 'Y') {
      if ($sameitem ne $ar->{$form->{sort}}) {
	&ar_subtotal;
      }
    }
    
    $column_data{netamount} = "<td align=right>&nbsp;" . $form->format_amount(\%myconfig, $ar->{netamount}) . "</td>";
    $column_data{tax} = "<td align=right>&nbsp;" . $form->format_amount(\%myconfig, $ar->{amount} - $ar->{netamount}) . "</td>";
    $column_data{amount} = "<td align=right>&nbsp;" . $form->format_amount(\%myconfig, $ar->{amount}) . "</td>";
    $column_data{paid} = "<td align=right>&nbsp;" . $form->format_amount(\%myconfig, $ar->{paid}) . "</td>";
    
    $subtotalnetamount += $ar->{netamount};
    $subtotalamount += $ar->{amount};
    $subtotalpaid += $ar->{paid};
    
    $totalnetamount += $ar->{netamount};
    $totalamount += $ar->{amount};
    $totalpaid += $ar->{paid};
    
    $column_data{transdate} = "<td>$ar->{transdate}&nbsp;</td>";
    $column_data{datepaid} = "<td>$ar->{datepaid}&nbsp;</td>";
    $column_data{duedate} = "<td>$ar->{duedate}&nbsp;</td>";

if ($ar->{invoice}) {
  $column_data{invnumber} = "<td><a href=is.pl?action=edit&id=$ar->{id}&path=$form->{path}&login=$form->{login}&callback=$callback>$ar->{invnumber}</a></td>";
} else {
  $column_data{invnumber} = "<td><a href=$form->{script}?action=edit&id=$ar->{id}&path=$form->{path}&login=$form->{login}&callback=$callback>$ar->{invnumber}</a></td>";
}

  $column_data{name} = "<td>$ar->{name}</td>";
  $column_data{terms} = "<td>&nbsp;$ar->{terms}</td>";

  print "<tr>";

  map { print "$column_data{$_}\n" } @column_index;

print "
</tr>
";

  }

  if ($form->{l_subtotal} eq 'Y') {
    &ar_subtotal;
  }

  # print totals
  print qq|<tr bgcolor=000000>|;

  map { $column_data{$_} = "<td>&nbsp;</td>" } @column_index;
  
  $column_data{netamount} = "<th align=right><font color=ffffff>&nbsp;" . $form->format_amount(\%myconfig, $totalnetamount) . "</th>";
  $column_data{tax} = "<th align=right><font color=ffffff>&nbsp;" . $form->format_amount(\%myconfig, $totalamount - $totalnetamount) . "</th>";
  $column_data{amount} = "<th align=right><font color=ffffff>&nbsp;" . $form->format_amount(\%myconfig, $totalamount) . "</th>";
  $column_data{paid} = "<th align=right><font color=ffffff>&nbsp;" . $form->format_amount(\%myconfig, $totalpaid) . "</th>";

  map { print "$column_data{$_}\n" } @column_index;

  print qq|</tr>

</table>

<p>|.$locale->text('Click on table headers to sort table').qq|
<br>|.$locale->text('Click on invoice number to edit transaction');

}


sub ar_subtotal {

  map { $column_data{$_} = "<td>&nbsp;</td>" } @column_index;
  
  $column_data{tax} = "<th align=right><font color=ffffff>&nbsp;" . $form->format_amount(\%myconfig, $subtotalamount - $subtotalnetamount) . "</font></th>";
  $column_data{amount} = "<th align=right><font color=ffffff>&nbsp;" . $form->format_amount(\%myconfig, $subtotalamount) . "</font></th>";
  $column_data{paid} = "<th align=right><font color=ffffff>&nbsp;" . $form->format_amount(\%myconfig, $subtotalpaid) . "</font></th>";
  
  $subtotalnetamount = 0;
  $subtotalamount = 0;
  $subtotalpaid = 0;

  $sameitem = $ar->{$form->{sort}};
  
  print "<tr bgcolor=333366>";

  map { print "$column_data{$_}\n" } @column_index;

print "
</tr>
";
 
}


sub receipts {

  # split customer
  ($customer, $form->{customer_id}) = split(/--/, $form->{customer});
  
  AR->receipts(\%myconfig, \%$form);

  # construct href
  $href = "$form->{script}?action=receipts&login=$form->{login}&path=$form->{path}&transdatefrom=$from->{transdatefrom}&transdateto=$form->{transdateto}&open=$form->{open}&invnumber=" . $form->escape($form->{invnumber}) . "&customer=" . $form->escape($form->{customer});
  
  @column_index = $form->sort_columns(qw(invnumber transdate name));
  
  $column_header{transdate} = "<th><a href=$href&sort=transdate><font color=ffffff>".$locale->text('Date')."</a></th>";
  $column_header{invnumber} = "<th><a href=$href&sort=invnumber><font color=ffffff>".$locale->text('Invoice')."</a></th>";
  $column_header{name} = "<th><a href=$href&sort=name><font color=ffffff>".$locale->text('Customer')."</a></th>";


  $form->{title} = $locale->text('Process Receipts');
  
  $form->header;
    
  print qq|
<h2>$form->{title}</h2>

<form action=$form->{script} method=post>

<table border=0>

<tr bgcolor=336666>
|;

map { print "$column_header{$_}\n" } @column_index;

  print qq|
<th><font color=ffffff>|.$locale->text('Amount').qq|</th>
</tr>
|;

  # construct callback
  $customer = $form->{customer};
  $customer =~ s/&/_/g;
  $callback = $form->escape("$form->{script}?action=receipts&transdatefrom=$from->{transdatefrom}&transdateto=$form->{transdateto}&open=$form->{open}&path=$form->{path}&login=$form->{login}&sort=$form->{sort}&invnumber=$form->{invnumber}&customer=$customer");
 
 
  foreach $accno (sort keys %{$form->{paidaccount}}) {
    $paidaccounts .= qq|<option>$form->{paidaccount}{$accno}\n|;
  }
 
  foreach $ar (@{ $form->{AR} }) {
    $i++;
    $amount = $form->format_amount(\%myconfig, $ar->{amount});
    $paid = $form->format_amount(\%myconfig, $ar->{paid});

    print "<tr>";
    
    $column_data{transdate} = qq|<td><input type=hidden name="transdate_$i" value=$ar->{transdate}>$ar->{transdate}</td>|;

if ($ar->{invoice}) {
  $column_data{invnumber} = "<td><a href=is.pl?action=edit&id=$ar->{id}&path=$form->{path}&login=$form->{login}&callback=$callback>$ar->{invnumber}</a></td>";
} else {
  $column_data{invnumber} = "<td><a href=$form->{script}?action=edit&id=$ar->{id}&path=$form->{path}&login=$form->{login}&callback=$callback>$ar->{invnumber}</a></td>";
}

    $column_data{name} = "<td>$ar->{name}</td>";

    map { print "$column_data{$_}\n" } @column_index;

    print qq|
<td align=right>$amount</td>
</tr>
<tr>
|;
  
      
    # arrange data for second row
    $column_data{invnumber} = qq|<th>|.$locale->text('RECEIVED').qq|</th>|;
    $column_data{transdate} = qq|<td><font size=-1><input name="datepaid_$i" size=10 value=$ar->{datepaid}></font></td>|;
    
    
    $column_data{name} = qq|<td><select name="accno_$i">\n|;
    $column_data{name} .= $paidaccounts;
    
    
    # check if the accno matches one of the accounts
    if ($ar->{paidaccno}) {
      $column_data{name} =~ s/<option>$form->{paidaccount}{$ar->{paidaccno}}/<option selected>$form->{paidaccount}{$ar->{paidaccno}}/;
    }

    $column_data{name} .= qq|</select></td>|;
    
    map { print "$column_data{$_}\n" } @column_index;

    print qq|
<td><font size=-1><input name="paid_$i" size=9 value=$paid></font></td>
</tr>

<input type=hidden name="id_$i" value=$ar->{id}>
<input type=hidden name="amount_$i" value=$ar->{amount}>
<input type=hidden name="AR_$i" value=$ar->{AR_account}>
<input type=hidden name="old_accno_$i" value=$ar->{paidaccno}>
<input type=hidden name="old_paid_$i" value=$ar->{paid}>

<tr><td colspan=4><hr size=3 noshade></td></tr>
|;

  }
  
  print qq|
</table>

<input type=hidden name=rowcount value=$i>

<p>
<input type=hidden name=path value=$form->{path}>
<input type=hidden name=login value=$form->{login}>

<input class=submit type=submit name=action value="|.$locale->text('Process Receipts').qq|">

</form>

<p>|.$locale->text('Click on table headers to sort table').qq|
<br>|.$locale->text('Click on invoice number to edit transaction');

}


sub process_receipts {

  # construct callback
  $form->{callback} = "$form->{script}?action=search&nextsub=receipts&path=$form->{path}&login=$form->{login}";

  AR->process_receipts(\%myconfig, \%$form);
  $form->redirect;

}


