#=====================================================================
# SQL-Ledger, Accounting
# Copyright (c) 2001
#
#  Author: Dieter Simader
#   Email: dsimader@sql-ledger.org
#     Web: http://www.sql-ledger.org
#
#  Contributors:
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#======================================================================
#
# Inventory Control module
#
# CHANGE LOG:
#   DS. 2000-07-04  Created
#   DS. 2001-06-26  Added assemblies
#   DS. 2001-07-25  added number formatting
#
#======================================================================


use DBI;
use SL::Form;
use SL::IC;

$form = new Form;


if ($form->{action}) {
  require("$userspath/$form->{login}.conf");
  $myconfig{dbpasswd} = unpack 'u', $myconfig{dbpasswd};
  map { $form->{$_} = $myconfig{$_} } qw(stylesheet charset);

  $locale = new Locale $myconfig{countrycode}, "ic";
  &{ $locale->findsub($form->{action}) };
} else {
  $form->error("action= not defined!");
}

# end of main



sub add {

  $form->{title} = $locale->text('Add ' . ucfirst $form->{item});

  $form->{callback} = "$form->{script}?action=add&item=$form->{item}&path=$form->{path}&login=$form->{login}";

  &link_part;
  &display_part;
  
}


sub search {

  $form->{title} = $locale->text('Search for Inventory Item');
  
  $form->header;
  
  print qq|

<h2>$form->{title}</h2>

<form method=post action=$form->{script}>

<table border=1>
<tr valign=top>
<td>
<table>

<tr>
  <th align=right>|.$locale->text('Number').qq|</th>
  <td><input name=partnumber size=20></td>
</tr>

<tr>
  <th align=right>|.$locale->text('Description').qq|</th>
  <td><input name=description size=35></td>
</tr>

<tr>
  <td colspan=2>
  <input name=searchitems class=radio type=radio value=all checked> <b>|.$locale->text('All').qq|
  <input name=searchitems class=radio type=radio value=parts> <b>|.$locale->text('Parts').qq|</b> 
  <input name=searchitems class=radio type=radio value=assemblies> <b>|.$locale->text('Assemblies').qq|</b> 
  <input name=searchitems class=radio type=radio value=services> <b>|.$locale->text('Services').qq|</b> 
  <input name=searchitems class=radio type=radio value=onhand> <b>|.$locale->text('On Hand').qq|</b>
  </td>
</tr>
  
</table>
</td>

</tr>
</table>

<p>
<input type=hidden name=path value=$form->{path}>
<input type=hidden name=login value=$form->{login}>

<input class=submit type=submit name=action value="|.$locale->text('Search for Item').qq|">
</form>

<p>|.$locale->text('To display all items, leave fields blank');

}


sub search_for_item {

  # setup $form->{sort}
  unless ($form->{sort}) {
    if ($form->{description} && !($form->{partnumber})) {
      $form->{sort} = "description";
    } else {
      $form->{sort} = "partnumber";
    }
  }
  
  $callback = "$form->{script}?action=search_for_item&path=$form->{path}&login=$form->{login}";

  IC->all_parts(\%myconfig, \%$form);
  
  if ($form->{partnumber}) {
    $callback .= "&partnumber=$form->{partnumber}";
  }
  if ($form->{description}) {
    $callback .= "&description=$form->{description}";
  }

  if ($form->{searchitems} ne 'all') {
    $callback .= "&searchitems=$form->{searchitems}";
  }

  @column_index = $form->sort_columns(qw(partnumber description onhand unit sellprice));

  $column_header{partnumber} = qq|<th><a href=$callback&sort=partnumber><font color=ffffff>|.$locale->text('Number').qq|</a></th>|;
  $column_header{description} = qq|<th><a href=$callback&sort=description><font color=ffffff>|.$locale->text('Description').qq|</a></th>|;
  $column_header{onhand} = qq|<th><font color=ffffff>|.$locale->text('On Hand').qq|</th>|;
  $column_header{unit} = qq|<th><font color=ffffff>|.$locale->text('Unit').qq|</th>|;
  $column_header{sellprice} = qq|<th><font color=ffffff>|.$locale->text('Sell Price').qq|</th>|;

  
  $form->{title} = $locale->text('Inventory');
  
  $form->header;

  print qq|
<h2>$form->{title}</h2>

<table border=1>
  <tr bgcolor=336666>
|;

  map { print "$column_header{$_}\n" } @column_index;
  
  print qq|
  </tr>
  |;


  # add order to callback and escape
  $callback = $form->escape($callback . "&sort=$form->{sort}");
  
  foreach $ref (@{ $form->{parts} }) {
    $column_data{sellprice} = "<td align=right>&nbsp;" . $form->format_amount(\%myconfig, $ref->{sellprice}) . "</td>";
    $column_data{onhand} = "<td align=right>&nbsp;".$form->format_amount(\%myconfig, $ref->{onhand}, "", "nat")."</td>";
    $column_data{unit} = "<td>&nbsp;$ref->{unit}";
    $column_data{partnumber} = "<td><a href=$form->{script}?action=edit&id=$ref->{id}&path=$form->{path}&login=$form->{login}&callback=$callback>$ref->{partnumber}</td>";
    $column_data{description} = "<td>$ref->{description}</td>";
    
    print "<tr>";

    map { print "$column_data{$_}\n" } @column_index;

    print qq|</tr>|;

  }

  print qq|
</table>

<p>|.$locale->text('Click on "Number" to edit').qq|
|;

}


sub edit {

  IC->get_part(\%myconfig, \%$form);

  $form->{title} = $locale->text('Edit ' . ucfirst $form->{item});

  &link_part;
  
  if ($form->{assemblycallback}) {
    $form->{callback} = $form->{assemblycallback};
    &check_form;
  }
  
  &display_part;
  
}


sub edit_a {

  $form->{assemblycallback} = "";
  &edit;

}


sub link_part {

  IC->create_links("IC", \%myconfig, \%$form);

  # parts and assemblies are equal
  $item = $form->{item};
  if ($form->{item} eq 'assembly') {
    $item = 'part';
  }
  
  # build the popup menus
  foreach $key (keys %{ $form->{IC_links} }) {
    foreach $ref (@{ $form->{IC_links}{$key} }) {
      # if this is a tax field
      if ($key =~ /IC_tax/) {
	if ($key =~ /$item/) {
	  $form->{taxaccounts} .= "$ref->{accno} ";
	  $form->{"IC_tax_$ref->{accno}_description"} = "$ref->{accno}--$ref->{description}";
	  
	  if ($form->{id}) {
	    if ($form->{amount}{$ref->{accno}}) {
	      $form->{"IC_tax_$ref->{accno}"} = "checked";
	    }
	  } else {
	    $form->{"IC_tax_$ref->{accno}"} = "checked";
	  }
	}
      } else {
	$form->{"select$key"} .= "<option>$ref->{accno}--$ref->{description}\n";
	if ($form->{amount}{$ref->{accno}}) {
	  $form->{$key} = "$ref->{accno}--$ref->{description}";
	}
	
      }
    }
  }
  chop $form->{taxaccounts};

  if (($form->{item} eq "part") || ($form->{item} eq "assembly")) {
    $form->{selectIC_income} = $form->{selectIC_sale};
    $form->{selectIC_expense} = $form->{selectIC_cogs};
    $form->{IC_income} = $form->{IC_sale};
    $form->{IC_expense} = $form->{IC_cogs};
  }
  
}


sub form_header {

  $form->{listprice} = $form->format_amount(\%myconfig, $form->{listprice});
  $form->{sellprice} = $form->format_amount(\%myconfig, $form->{sellprice});
  
  foreach $item (qw(partnumber description unit notes)) {
    $form->{$item} =~ s/"/&quot;/g;
  }
  
  foreach $item (split / /, $form->{taxaccounts}) {
    $form->{"IC_tax_$item"} = ($form->{"IC_tax_$item"}) ? "checked" : "";
  }

  # set option
  foreach $item (qw(IC IC_income IC_expense)) {
    $form->{"select$item"} =~ s/option selected/option/;
    $form->{"select$item"} =~ s/option>$form->{$item}/option selected>$form->{$item}/;
  }

  $charge = qq|
<tr>
  <td colspan=2>
  <b>|.$locale->text('List Price').qq|</b> <input name=listprice size=9 value=$form->{listprice}>
  <b>|.$locale->text('Sell Price').qq|</b> <input name=sellprice size=9 value=$form->{sellprice}>|;


  if ($form->{item} eq "part") {

    $inventory = qq|
<tr>
  <th>|.$locale->text('Link Accounts').qq|:</th>
  <td>

  <b>|.$locale->text('Inventory').qq|</b>&nbsp;<select name=IC>
$form->{selectIC}</select>
<input name=selectIC type=hidden value="$form->{selectIC}">

<br><b>|.$locale->text('Sales').qq|</b>&nbsp;<select name=IC_income>
$form->{selectIC_income}</select>
<input name=selectIC_income type=hidden value="$form->{selectIC_income}">

<br><b>|.$locale->text('COGS').qq|</b>&nbsp;<select name=IC_expense>
$form->{selectIC_expense}</select>
<input name=selectIC_expense type=hidden value="$form->{selectIC_expense}">
|;

    $weight = qq|
<tr>
  <td><b>|.$locale->text('Weight').qq|</b> 
  <input name=weight size=5 value=|.$form->format_amount(\%myconfig, $form->{weight}, "", "nat").qq|> <b>$form->{weightunit}</b></td>
  <td><b>|.$locale->text('On Hand').qq| </b>|.$form->format_amount(\%myconfig, $form->{onhand}, "", "nat").qq|
  </td>
</tr>|;

    $charge .= qq|<b> |.$locale->text('Last Cost').qq|</b> |.$form->format_amount(\%myconfig, $form->{lastcost}).qq| <b>$form->{nativecurr}</b>|;
    }


  if ($form->{item} eq "assembly") {
    $inventory = qq|
<tr>
  <th>|.$locale->text('Link Accounts').qq|:</th>
  <td>

<b>|.$locale->text('Income').qq|</b>&nbsp;<select name=IC_income>
$form->{selectIC_income}</select>
<input name=selectIC_income type=hidden value="$form->{selectIC_income}">

|;
    
    $charge .= qq| <b>$form->{nativecurr}</b>|;
  }

 
  if ($form->{item} eq "service") {
    $inventory = qq|
<tr>
  <th>|.$locale->text('Link Accounts').qq|:</th>
  <td>

<b>|.$locale->text('Income').qq|</b>&nbsp;<select name=IC_income>
$form->{selectIC_income}</select>
<input name=selectIC_income type=hidden value="$form->{selectIC_income}">

<br><b>|.$locale->text('Expense').qq|</b>&nbsp;<select name=IC_expense>
$form->{selectIC_expense}</select>
<input name=selectIC_expense type=hidden value="$form->{selectIC_expense}">
|;
    
    $charge .= qq|<b> |.$locale->text('Last Cost').qq|</b> |.$form->format_amount(\%myconfig, $form->{lastcost}).qq| <b>$form->{nativecurr}</b>|;
  }

  # add tax fields to $inventory
  foreach $item (split / /, $form->{taxaccounts}) {
    $inventory .= qq|
<br><input class=checkbox type=checkbox name="IC_tax_$item" value=1 $form->{"IC_tax_$item"}>
&nbsp;<b>$form->{"IC_tax_${item}_description"}</b>
<input type=hidden name=IC_tax_${item}_description value=$form->{"IC_tax_${item}_description"}>
|;
  }

  $inventory .= qq|
  </td>
</tr>
|;

  $charge .= qq|
<input name=taxaccounts type=hidden value="$form->{taxaccounts}">
</td></tr>|;

 
  unless ($form->{item} eq 'service') {
    $rop = qq|<b>|.$locale->text('ROP').qq|</b> 
<input name=rop size=5 value=|.$form->format_amount(\%myconfig, $form->{rop}, "", "nat").qq|>|;
  }

# class=submit type=submit $locale->text('Add Part')
# class=submit type=submit $locale->text('Add Service')
# class=submit type=submit $locale->text('Add Assembly')

# class=submit type=submit $locale->text('Edit Part')
# class=submit type=submit $locale->text('Edit Service')
# class=submit type=submit $locale->text('Edit Assembly')


  $form->header;

  print qq|

<h2>$form->{title}</h2>

<form method=post action=$form->{script}>

<input name=id type=hidden value=$form->{id}>
<input name=item type=hidden value=$form->{item}>
<input name=title type=hidden value="$form->{title}">

<input name=makemodel type=hidden value="$form->{makemodel}">
<input name=alternate type=hidden value="$form->{alternate}">
<input name=onhand type=hidden value=$form->{onhand}>

<table border=1>
<tr valign=top>
<td>
<table callspacing=0 cellpadding=0>

<tr>
  <td><b>|.$locale->text('Number').qq|</b>
  <br><input name=partnumber size=20 value="$form->{partnumber}"></td>
  <td><b>|.$locale->text('Description').qq|</b>
  <br><input name=description size=40 value="$form->{description}"></td>
</tr>

$weight

<tr>
  <td><b>|.$locale->text('Unit of measure').qq|</b> 
  <input name=unit size=5 maxsize=5 value="$form->{unit}"></td>
  <td><b>|.$locale->text('Price Updated').qq| ($myconfig{dateformat})</b>
  <input name=priceupdate size=10 value=$form->{priceupdate}>
  </td>
</tr>

$charge

<tr valign=top>
  <td>$rop</td>
  
  <td>
  <b>|.$locale->text('Notes').qq|</b>
  <br><textarea name=notes rows=3 cols=35>$form->{notes}</textarea>
  </td>
</tr>

$inventory

 
|;
}


sub form_footer {

# class=submit type=submit $locale->text('Delete Part')
# class=submit type=submit $locale->text('Delete Service')
# class=submit type=submit $locale->text('Delete Assembly')
# class=submit type=submit $locale->text('Save Part')
# class=submit type=submit $locale->text('Save Service')
# class=submit type=submit $locale->text('Save Assembly')

  if ($form->{item} eq "assembly") {

    $form->{weight} = $form->format_amount(\%myconfig, $form->{weight}, "", "nat");
    
    print qq|
<tr>
  <th colspan=2 align=left>|.$locale->text('Weight').qq| $form->{weight} <b>$form->{weightunit}</b></th>
<th align=right>|.$locale->text('Total').qq|</th>
<th align=right>|.$form->format_amount(\%myconfig, $form->{assemblytotal}).qq|</th></tr>

</table>
</td>
</tr>

<input type=hidden name=weight value=$form->{weight}>

|;
  }

 
  print qq|

<input type=hidden name=path value=$form->{path}>
<input type=hidden name=login value=$form->{login}>

<input name=callback type=hidden value="$form->{callback}">

</table>
</td>

</tr>
</table>

<br>|;

  unless ($form->{item} eq "service") {
    print qq|<input class=submit type=submit name=action value="|.$locale->text('Update Form').qq|">
|;
  }
  
  print qq|
<input class=submit type=submit name=action value="|.$locale->text('Save ' . ucfirst $form->{item}).qq|">
|;


  if ($form->{id}) {
    print qq|<input class=submit type=submit name=action value="|.$locale->text('Delete '. ucfirst $form->{item}).qq|">
|;
  }

  print "</form>";

}


sub display_part {

  &form_header;

  $form->{rowcount}++;
  
  if ($form->{item} eq "part") {
    map { &makemodel_row($_) } (1 .. $form->{rowcount});
  }

  if ($form->{item} eq "assembly") {
    map { &assembly_row($_) } (1 .. $form->{rowcount});
  }

  &form_footer;
}


sub makemodel_row {
  my $i = shift;

  $form->{"make_$i"} =~ s/"/&quot;/g;
  $form->{"model_$i"} =~ s/"/&quot;/g;

  if ($i == 1) {
    print qq|
    <tr>
    <td><b>|.$locale->text('Make').qq|</b>
    <br><input name="make_$i" size=20 value="$form->{"make_$i"}"></td>
    <td colspan=2><b>|.$locale->text('Model').qq|</b>
    <br><input name="model_$i" size=40 value="$form->{"model_$i"}"></td>
    </tr>
    <input type=hidden name=rowcount value=$i>
    |;
  } else {
    print qq|
    <tr>
    <td><input name="make_$i" size=20 value="$form->{"make_$i"}"></td>
    <td colspan=2><input name="model_$i" size=40 value="$form->{"model_$i"}"></td>
    </tr>
    <input type=hidden name=rowcount value=$i>
    |;
  }
}


sub assembly_row {
  my $i = shift;

  $form->{"partnumber_$i"} =~ s/"/&quot;/g;

  if ($i == 1) {
    $form->{assemblytotal} = 0;
    $form->{weight} = 0;
    
    print qq|
<tr bgcolor=336666><th colspan=3><font color=ffffff>|.$locale->text('Individual Items').qq|</font></th></tr>

<tr><td colspan=3>
<table>
  <tr>
    <th>|.$locale->text('Qty').qq|</th>
    <th>|.$locale->text('Number').qq|</th>
    <th>|.$locale->text('Description').qq|</th>
    <th>|.$locale->text('Sell Price').qq|</th>
  </tr>
|;
  }
  
  $linetotal = sprintf("%.2f", $form->{"sellprice_$i"} * $form->{"qty_$i"});
  $form->{assemblytotal} += $linetotal;
  $form->{weight} += $form->{"weight_$i"} * $form->{"qty_$i"};
  
  $form->{"qty_$i"} = $form->format_amount(\%myconfig, $form->{"qty_$i"}, "", "nat");
  
  print qq|
<tr>
  <td><input name="qty_$i" size=5 value="$form->{"qty_$i"}"></td>
|;

  if (($form->{rowcount} >= 1) && ($form->{rowcount} == $i)) {
    print qq|
  <td><input name="partnumber_$i" size=15 value="$form->{"partnumber_$i"}"></td>
  <td><input name="description_$i" size=30 value="$form->{"description_$i"}"></td>
|;
  } else {
    # double escape assemblycallback
    $form->{assemblycallback} = $form->escape($form->{callback});
    
    $href = qq|$form->{script}?action=edit_a&id=$form->{"id_$i"}&path=$form->{path}&login=$form->{login}&callback=|.$form->escape("$form->{script}?action=edit&id=$form->{id}&path=$form->{path}&login=$form->{login}&assemblycallback=$form->{assemblycallback}");
    
    print qq|
  <td><input type=hidden name="partnumber_$i" value="$form->{"partnumber_$i"}">
      <a href=$href>$form->{"partnumber_$i"}</a></td>
  <td><input type=hidden name="description_$i" value="$form->{"description_$i"}">$form->{"description_$i"}</td>
|;
  }

  $linetotal = $form->format_amount(\%myconfig, $linetotal);

  print qq|
  <td align=right>$linetotal</td>
</tr>
  <input type=hidden name="id_$i" value=$form->{"id_$i"}>
  <input type=hidden name="sellprice_$i" value=$form->{"sellprice_$i"}>
  <input type=hidden name="weight_$i" value=$form->{"weight_$i"}>
  <input type=hidden name=rowcount value=$i>
|;

 
}


sub update_form {

  $i = $form->{rowcount};
  
  # check if there is a part number
  $form->isblank("partnumber", $locale->text(ucfirst $form->{item} . " Number missing!"));
  
  if ($form->{item} eq "assembly") {
    # if last row is empty check the form otherwise retrieve item
    if (($form->{"partnumber_$i"} eq "") && ($form->{"description_$i"} eq "")) {
      
      &check_form;
      
    } else {

      IC->assembly_item(\%myconfig, \%$form);

      $rows = scalar @{ $form->{item_list} };
      if ($rows) {
	$form->{"qty_$i"}           = 1 unless ($form->{"qty_$i"});

	if ($rows > 1) {
	  &select_item;
	  exit;
	} else {
	  map { $form->{item_list}[$i]{$_} =~ s/"/&quot;/g } qw(partnumber description);
	  map { $form->{"${_}_$i"} = $form->{item_list}[0]{$_} } keys %{ $form->{item_list}[0] };

	  &check_form;

	}

      } else {
	$form->error($locale->text("Item not on file!"));
      }
    }
  }
  
  if ($form->{item} eq "part") {
    &check_form;
  }

  &display_part;

}


sub check_form {
  
  my @a = ();
  my $count = 0;
  my @flds = qw(id partnumber description qty sellprice make model);
  my $sellprice = 0;
  
  # remove any empty numbers, makes or model rows
  for my $i (1 .. ($form->{rowcount})) {
    if (($form->{"make_$i"} ne "") || ($form->{"model_$i"} ne "")) {
      push @a, {};
      my $j = $#a;

      map { $a[$j]->{$_} = $form->{"${_}_$i"} } @flds;
      $count++;
    }
    
    if (($form->{"qty_$i"}) && (($form->{"partnumber_$i"} ne "") || ($form->{"description_$i"} ne ""))) {
      push @a, {};
      my $j = $#a;

      map { $a[$j]->{$_} = $form->{"${_}_$i"} } @flds;
      $sellprice += ($form->{"qty_$i"} * $form->{"sellprice_$i"});
      $count++;
    }
  }

  for $i (1 .. $count) {
    $j = $i - 1;
    map { $form->{"${_}_$i"} = $a[$j]->{$_} } @flds;
  }

  for $i ($count + 1 .. $form->{rowcount}) {
    map { delete $form->{"${_}_$i"} } @flds;
  }

  $form->{rowcount} = $count;

  if ($form->{item} eq 'assembly') {
    $form->{sellprice} = $sellprice;
    IC->save(\%myconfig, \%$form);
  }
  
}


sub select_item {

  # list items with radio button on a form
  $form->header;

  print qq|
<h2>|.$locale->text('Select from one of the items below').qq|</h2>

<form method=post action=$form->{script}>

<table border=1>
  <tr><td>

  <table border=0>

    <tr bgcolor=336666>
      <th>&nbsp;</th>
      <th><font color=ffffff>|.$locale->text('Number').qq|</th>
      <th><font color=ffffff>|.$locale->text('Description').qq|</th>
      <th><font color=ffffff>|.$locale->text('Sell Price').qq|</th>
|;

  my $i = 0;
  foreach $ref (@{ $form->{item_list} }) {
    $checked = ($i++) ? "" : "checked";

    map { $ref->{$_} =~ s/"/&quot;/g } qw(partnumber description);
		    
    print qq|
<tr>
  <td><input name=ndx class=radio type=radio value=$i $checked></td>
  <td><input name="new_partnumber_$i" type=hidden value="$ref->{partnumber}">$ref->{partnumber}</td>
  <td><input name="new_description_$i" type=hidden value="$ref->{description}">$ref->{description}</td>
  <td align=right><input name="new_sellprice_$i" type=hidden value=$ref->{sellprice}>|.$form->format_amount(\%myconfig, $ref->{sellprice}).qq|</td>
</tr>

<input name="new_weight_$i" type=hidden value=$ref->{weight}>
<input name="new_id_$i" type=hidden value=$ref->{id}>

|;

  }

  print qq|
  </table>
  </td></tr>
</table>
|;


  # delete unneccessary variables
  delete $form->{action};
  delete $form->{item_list};
					        
  # save all other form variables
  foreach $key (keys %${form}) {
    $form->{$key} =~ s/"/&quot;/g;
    print qq|<input name=$key type=hidden value="$form->{$key}">\n|;
  }

  print qq|
  <input type=hidden name=nextsub value=item_selected>
  <input class=submit type=submit name=action value="|.$locale->text('Continue').qq|">
</form>|;

}


sub continue {

  &{ $form->{nextsub} };

}


sub item_selected {

  # replace the last row with the checked row
  $i = $form->{rowcount};

  # index for new item
  $j = $form->{ndx};

  map { $form->{"${_}_$i"} = $form->{"new_${_}_$j"} } qw(id partnumber description sellprice listprice lastcost weight);

  &check_form;
  &display_part;

}


sub save_part {
  &save;
}

sub save_service {
  &save;
}

sub save_assembly {
  &save;
}


sub save {

  # check if there is a part number
  $form->isblank("partnumber", $locale->text(ucfirst $form->{item} . " Number missing!"));

# $locale->text('Part Number missing!')
# $locale->text('Service Number missing!')
# $locale->text('Assembly Number missing!')

  $form->isposted(IC->save(\%myconfig, \%$form));

}


sub delete_part {
  
  if ($form->{onhand}) {
    $form->error($locale->text("Cannot delete part, on hand quantity must be zero!"));
  } else {
    &yes_delete;
  }
  
}


sub delete_service {

  &yes_delete;
  
}


sub delete_assembly {

  &yes_delete;

}


sub yes_delete {

  $rc = IC->delete(\%myconfig, \%$form);
  
  if ($rc == 2) {
    $form->error($locale->text("Cannot delete item already invoiced"));
  }
  
  if ($rc == 3) {
    $form->error($locale->text("Cannot delete item which is part of an assembly"));
  }
    
  $form->isdeleted($rc);

}


