#=====================================================================
# SQL-Ledger Accounting
# Copyright (C) 2001
#
#  Author: Dieter Simader
#   Email: dsimader@sql-ledger.org
#     Web: http://www.sql-ledger.org
#
#  Contributors: Martin Lillepuu <martin@lillepuu.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#======================================================================
#
# module for Chart of Accounts, Income Statement and Balance Sheet
# search and edit transactions posted by the GL, AR and AP
# 
# CHANGE LOG:
#   DS. 2000-05-22  Created
#   ML. 2001-02-16  fixed sorting by date when international date format used
#   DS. 2001-07-25  added number formatting
#
#======================================================================

use DBI;
use SL::Form;
use SL::CA;


$form = new Form;

require("$userspath/$form->{login}.conf");
$myconfig{dbpasswd} = unpack 'u', $myconfig{dbpasswd};
map { $form->{$_} = $myconfig{$_} } qw(stylesheet charset);

$locale = new Locale $myconfig{countrycode}, "ca";

if ($form->{action}) {
  &{ $locale->findsub($form->{action}) };
} else {
  &list_accounts;
}

# end of main


# this is for our long dates
# $locale->text('January')
# $locale->text('February')
# $locale->text('March')
# $locale->text('April')
# $locale->text('May')
# $locale->text('June')
# $locale->text('July')
# $locale->text('August')
# $locale->text('September')
# $locale->text('October')
# $locale->text('November')
# $locale->text('December')

# this is for our short month
# $locale->text('Jan')
# $locale->text('Feb')
# $locale->text('Mar')
# $locale->text('Apr')
# $locale->text('May')
# $locale->text('Jun')
# $locale->text('Jul')
# $locale->text('Aug')
# $locale->text('Sep')
# $locale->text('Oct')
# $locale->text('Nov')
# $locale->text('Dec')



sub list_accounts {

  CA->all_accounts(\%myconfig, \%$form);

  @column_index = qw(accno description debit credit);

  $column_header{accno} = qq|<th><font color=ffffff>|.$locale->text('Account').qq|</th>\n|;
  $column_header{description} = qq|<th><font color=ffffff>|.$locale->text('Description').qq|</th>\n|;
  $column_header{debit} = qq|<th><font color=ffffff>|.$locale->text('Debit').qq|</th>\n|;
  $column_header{credit} = qq|<th><font color=ffffff>|.$locale->text('Credit').qq|</th>\n|;
  

  $form->{title} = $locale->text('Accounts List');
  
  $form->header;

  print qq|
  
<h2>$form->{title}</h2>

<table border=1>
<tr bgcolor=000000>|;

  map { print $column_header{$_} } @column_index;

  print qq|
</tr>
|;

  
  foreach $ca (@{ $form->{CA} }) {

    $href = qq|$form->{script}?path=$form->{path}&action=list&accno=$ca->{accno}&login=$form->{login}&description=| . $form->escape($ca->{description});
      
    if ($ca->{charttype} eq "H") {
      print qq|<tr bgcolor=336666>|;
      $column_data{accno} = "<th><font color=ffffff>$ca->{accno}</th>";
      $column_data{description} = "<th><font color=ffffff>$ca->{description}</th>";
    } else {
      $column_data{accno} = "<td><a href=$href>$ca->{accno}</a></td>";
      $column_data{description} = "<td>$ca->{description}</td>";
    }
      
    $column_data{debit} = "<td align=right>&nbsp;".$form->format_amount(\%myconfig, $ca->{debit})."</td>\n";
    $column_data{credit} = "<td align=right>&nbsp;".$form->format_amount(\%myconfig, $ca->{credit})."</td>\n";
    
    $totaldebit += $ca->{debit};
    $totalcredit += $ca->{credit};

    map { print $column_data{$_} } @column_index;

    print qq|
</tr>|;
  }

  $column_data{accno} = "<td>&nbsp;</td>";
  $column_data{description} = "<td>&nbsp;</td>";
  $column_data{debit} = "<th align=right><font color=ffffff>&nbsp;".$form->format_amount(\%myconfig, $totaldebit)."</font></th>";
  $column_data{credit} = "<th align=right><font color=ffffff>&nbsp;".$form->format_amount(\%myconfig, $totalcredit)."</font></th>";
  
  print "<tr bgcolor=000000>";

  map { print $column_data{$_} } @column_index;

  print qq|
</tr>

</table>

<br>|.$locale->text('Click on account number to display transactions')

}


sub list {

  $form->{title} = $locale->text('List Transactions');
  
  $form->header;
  
  $form->{description} =~ s/"/&quot;/g;
  
  print qq|
<h2>$form->{title}</h2>

<form method=post action=$form->{script}>

<input type=hidden name=accno value=$form->{accno}>
<input type=hidden name=description value="$form->{description}">
<input type=hidden name=sort value=transdate>

<table border=1>
<tr valign=top>
<td>
<table>

<tr>
<th align=right>|.$locale->text('From').qq| ($myconfig{dateformat})</th><td><input name=fromdate size=10></td>
<th align=right>|.$locale->text('To').qq|</th><td><input name=todate size=10></td>
</tr>

<tr>
  <th align=right>|.$locale->text('Include in Report').qq|</th>
  <td colspan=3><input name=l_subtotal lass=checkbox type=checkbox value=Y>&nbsp;|.$locale->text('Subtotal').qq|</td>
</table>
</td>

</tr>
</table>

<p>
<input type=hidden name=login value=$form->{login}>
<input type=hidden name=path value=$form->{path}>

<p><input class=submit type=submit name=action value="|.$locale->text('List Transactions').qq|">
</form>

<p>|.$locale->text('To display all transactions, leave fields blank');

}


sub list_transactions {

  CA->all_transactions(\%myconfig, \%$form);

  # construct href
  $href = "$form->{script}?path=$form->{path}&action=list_transactions&accno=$form->{accno}&login=$form->{login}&fromdate=$form->{fromdate}&todate=$form->{todate}&l_subtotal=$form->{l_subtotal}&description=" . $form->escape($form->{description});

  # construct callback
  $callback = "$form->{script}?path=$form->{path}&action=list_transactions&accno=$form->{accno}&description=$form->{description}&login=$form->{login}&fromdate=$form->{fromdate}&todate=$form->{todate}&subtotal=$form->{l_subtotal}";

  # figure out which column comes first
  $column_header{transdate} = qq|<th><a href=$href&sort=transdate><font color=ffffff>|.$locale->text('Date').qq|</a></th>|;
  $column_header{source} = qq|<th><a href=$href&sort=source><font color=ffffff>|.$locale->text('Source').qq|</a></th>|;
  $column_header{description} = qq|<th><a href=$href&sort=description><font color=ffffff>|.$locale->text('Description').qq|</a></th>|;
  $column_header{debit} = qq|<th><font color=ffffff>|.$locale->text('Debit').qq|</th>|;
  $column_header{credit} = qq|<th><font color=ffffff>|.$locale->text('Credit').qq|</th>|;

  @column_index = $form->sort_columns(qw(transdate source description debit credit));
  
  $form->{title} = $locale->text('Account') .qq| $form->{accno} - $form->{description} |.$locale->text('Transactions');
  

  if ($form->{fromdate} || $form->{todate}) {
    unless ($form->{todate}) {
      $form->{todate} = $form->current_date(\%myconfig);
    }

    $fromdate = $locale->date(\%myconfig, $form->{fromdate}, 1);
    $todate = $locale->date(\%myconfig, $form->{todate}, 1);
    $form->{period} = qq|$fromdate |.$locale->text('To').qq| $todate|;
  } else {
    $form->{period} = $locale->date(\%myconfig, $form->current_date(\%myconfig),1);
  }
    
  
  $form->header;
  
  print qq|
<h2>$form->{title}</h2>

<b>$form->{period}</b>

<p>
<table border=1>
<tr bgcolor=336666>
|;

map { print "$column_header{$_}\n" } @column_index;

print qq|</tr>|;


  # add sort to callback
  $callback = $form->escape($callback . "&sort=$form->{sort}");

  $sameitem = $form->{CA}->[0]->{$form->{sort}};

  foreach $ca (@{ $form->{CA} }) {

    if ($form->{l_subtotal} eq 'Y') {
      if ($sameitem ne $ca->{$form->{sort}}) {
	&ca_subtotal;
      }
    }
    
    # construct link to source
    $href = "<a href=$ca->{module}.pl?path=$form->{path}&action=edit&id=$ca->{id}&login=$form->{login}&callback=$callback>$ca->{source}</a>";

    
    $column_data{debit} = "<td align=right>&nbsp;".$form->format_amount(\%myconfig, $ca->{debit})."</td>";
    $column_data{credit} = "<td align=right>&nbsp;".$form->format_amount(\%myconfig, $ca->{credit})."</td>";

    $subtotaldebit += $ca->{debit};
    $subtotalcredit += $ca->{credit};
    
    $totaldebit += $ca->{debit};
    $totalcredit += $ca->{credit};
    
    $column_data{transdate} = qq|<td>$ca->{transdate}</td>|;
    $column_data{source} = qq|<td>$href</td>|;
    $column_data{description} = qq|<td>$ca->{description}</td>|;
  
    print "<tr>";

    map { print $column_data{$_} } @column_index;

    print qq|</tr>|;

  }
 

  if ($form->{l_subtotal} eq 'Y') {
    &ca_subtotal;
  }
 

  map { $column_data{$_} = "<td>&nbsp;</td>" } @column_index;
  
  $column_data{debit} = "<th align=right><font color=ffffff>&nbsp;".$form->format_amount(\%myconfig, $totaldebit)."</font></th>";
  $column_data{credit} = "<th align=right><font color=ffffff>&nbsp;".$form->format_amount(\%myconfig, $totalcredit)."</font></th>";

  print qq|
<tr bgcolor=000000>
|;

  map { print $column_data{$_} } @column_index;
  
  print qq|
</tr>

</table>

</body>
</html>
|;
  
}


sub ca_subtotal {

  map { $column_data{$_} = "<td>&nbsp;</td>" } @column_index;
  
  $column_data{debit} = "<th align=right><font color=ffffff>&nbsp;" . $form->format_amount(\%myconfig, $subtotaldebit) . "</font></th>";
  $column_data{credit} = "<th align=right><font color=ffffff>&nbsp;" . $form->format_amount(\%myconfig, $subtotalcredit) . "</font></th>";
       
  $subtotaldebit = 0;
  $subtotalcredit = 0;

  $sameitem = $ca->{$form->{sort}};

  print "<tr bgcolor=333366>";

  map { print "$column_data{$_}\n" } @column_index;

  print "
</tr>
";

}

