#=====================================================================
# SQL-Ledger Accounting
# Copyright (c) 2001
#
#  Author: Dieter Simader
#   Email: dsimader@sql-ledger.org
#     Web: http://www.sql-ledger.org
#
#  Contributors:
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#======================================================================
#
# Genereal Ledger
#
# CHANGE LOG:
#   DS. 2000-07-04  Created
#   DS. 2001-06-20  added field selection for reports
#   DS. 2001-07-10  included AR and AP transactions in GL report
#   DS. 2001-07-25  added number formatting
#   DS. 2001-07-27  added sales and purchase journal reporting
#
#======================================================================


use DBI;
use SL::Form;
use SL::GL;

$form = new Form;


if ($form->{action}) {
  require("$userspath/$form->{login}.conf");
  $myconfig{dbpasswd} = unpack 'u', $myconfig{dbpasswd};
  map { $form->{$_} = $myconfig{$_} } qw(stylesheet charset);

  $locale = new Locale $myconfig{countrycode}, "gl";
  &{ $locale->findsub($form->{action}) };
} else {
  $form->error("action= not defined!");
}

# end of main


# this is for our long dates
# $locale->text('January')
# $locale->text('February')
# $locale->text('March')
# $locale->text('April')
# $locale->text('May')
# $locale->text('June')
# $locale->text('July')
# $locale->text('August')
# $locale->text('September')
# $locale->text('October')
# $locale->text('November')
# $locale->text('December')

# this is for our short month
# $locale->text('Jan')
# $locale->text('Feb')
# $locale->text('Mar')
# $locale->text('Apr')
# $locale->text('May')
# $locale->text('Jun')
# $locale->text('Jul')
# $locale->text('Aug')
# $locale->text('Sep')
# $locale->text('Oct')
# $locale->text('Nov')
# $locale->text('Dec')


sub add {

  $form->{title} = "Add";
  
  $form->{callback} = "$form->{script}?action=add&path=$form->{path}&login=$form->{login}";

  # we use this only to set a default date
  GL->transaction(\%myconfig, \%$form);
  
  $form->{rowcount} = 1;
  &display_form;
  
}


sub search {

  $form->{title} = $locale->text('Search for Transaction');
  
  $form->header;
  
  print qq|
<h2>$form->{title}</h2>

<form method=post action=$form->{script}>

<input type=hidden name=sort value=transdate>

<table border=1>
<tr valign=top>
<td>
<table>


<tr>
<th align=right>|.$locale->text('Source').qq|</th><td><input name=source size=20></td>
</tr>

<tr>
<th align=right>|.$locale->text('From').qq| ($myconfig{dateformat})</th><td><input name=datefrom size=10></td>
<th align=right>|.$locale->text('To').qq| <input name=dateto size=10></td>
</tr>

<tr>
<th align=right>|.$locale->text('Description').qq|</th><td colspan=3><input name=description size=50></td>
</tr>

<tr>
<th align=right>|.$locale->text('Include in Report').qq|</th>
<td colspan=3>
<input name="category" class=radio type=radio value=X checked>&nbsp;|.$locale->text('All').qq|
<input name="category" class=radio type=radio value=A>&nbsp;|.$locale->text('Asset').qq|
<input name="category" class=radio type=radio value=L>&nbsp;|.$locale->text('Liability').qq|
<input name="category" class=radio type=radio value=Q>&nbsp;|.$locale->text('Equity').qq|
<input name="category" class=radio type=radio value=I>&nbsp;|.$locale->text('Income').qq|
<input name="category" class=radio type=radio value=E>&nbsp;|.$locale->text('Expense').qq|
<br>
<input name="l_transdate" class=checkbox type=checkbox value=Y checked>&nbsp;|.$locale->text('Date').qq|
<input name="l_source" class=checkbox type=checkbox value=Y checked>&nbsp;|.$locale->text('Source').qq|
<input name="l_description" class=checkbox type=checkbox value=Y checked>&nbsp;|.$locale->text('Description').qq|
<input name="l_debit" class=checkbox type=checkbox value=Y checked>&nbsp;|.$locale->text('Debit').qq|
<input name="l_credit" class=checkbox type=checkbox value=Y checked>&nbsp;|.$locale->text('Credit').qq|
<input name="l_accno" class=checkbox type=checkbox value=Y checked>&nbsp;|.$locale->text('Account').qq|
<br>
<input name="l_subtotal" class=checkbox type=checkbox value=Y>&nbsp;|.$locale->text('Subtotal').qq|
</td></tr>

</table>
</td>

</tr>
</table>

<p>
<input type=hidden name=path value=$form->{path}>
<input type=hidden name=login value=$form->{login}>

<input class=submit type=submit name=action value="|.$locale->text('Search for Transaction').qq|">
</form>

<p>|.$locale->text('To display all transactions, leave fields blank').qq|

</body>
</html>
|;
}


sub search_for_transaction {

  GL->all_transactions(\%myconfig, \%$form);
  
  $callback = "$form->{script}?action=search_for_transaction&path=$form->{path}&login=$form->{login}";
 
  %acctype = ( 'A' => $locale->text('Asset'),
               'L' => $locale->text('Liability'),
	       'Q' => $locale->text('Equity'),
	       'I' => $locale->text('Income'),
	       'E' => $locale->text('Expense'),
	     );
  
  $form->{title} = $locale->text('General Ledger');
  
  unless ($form->{category} eq 'X') {
    $form->{title} .= " -> ".$locale->text($acctype{$form->{category}});
  }
  if ($form->{accno}) {
    $callback .= "&accno=$form->{accno}";
    $form->{title} .= "<br>".$locale->text('Account')." $form->{accno} $form->{account_description}";
  }
  if ($form->{source}) {
    $callback .= "&source=$form->{source}";
    $form->{title} .= "<br>".$locale->text('Source')." -> $form->{source}";
  }
  if ($form->{description}) {
    $callback .= "&description=$form->{description}";
    $form->{title} .= "<br>".$locale->text('Description')." -> $form->{description}";
  }
    
  if ($form->{datefrom}) {
    $callback .= "&datefrom=$form->{datefrom}";
    $form->{title} .= "<br>".$locale->text('From').": ".$locale->date(\%myconfig, $form->{datefrom}, 1);
  }
  if ($form->{dateto}) {
    $callback .= "&dateto=$form->{dateto}";
    $form->{title} .= ($form->{datefrom}) ? " " : "<br>";
    $form->{title} .= $locale->text('To').": ".$locale->date(\%myconfig, $form->{dateto}, 1);
  }


  @columns = $form->sort_columns(qw(transdate source description debit credit accno));
  if ($form->{accno}) {
    @columns = grep !/accno/, @columns;
  }

  $href = "$callback&sort=$form->{sort}";        # needed for accno
  
  foreach $item (@columns) {
    if ($form->{"l_$item"} eq "Y") {
      push @column_index, $item;

      # add column to href and callback
      $callback .= "&l_$item=Y";
      $href .= "&l_$item=Y";
    }
  }
  
  if ($form->{l_subtotal} eq 'Y') {
    $callback .= "&l_subtotal=Y";
    $href .= "&l_subtotal=Y";
  }

  $callback .= "&category=$form->{category}";
  $href .= "&category=$form->{category}";


  $column_header{transdate} = "<th><a href=$callback&sort=transdate><font color=ffffff>".$locale->text('Date')."</a></th>";
  $column_header{source} = "<th><a href=$callback&sort=source><font color=ffffff>".$locale->text('Source')."</a></th>";
  $column_header{description} = "<th><a href=$callback&sort=description><font color=ffffff>".$locale->text('Description')."</a></th>";
  $column_header{debit} = "<th><font color=ffffff>".$locale->text('Debit')."</th>";
  $column_header{credit} = "<th><font color=ffffff>".$locale->text('Credit')."</th>";
  $column_header{accno} = "<th><a href=$callback&sort=accno><font color=ffffff>".$locale->text('Account')."</a></th>";
 

  $form->header;

  print qq|
<h2>$form->{title}</h2>

<table border=1>
<tr bgcolor=336666>
|;

map { print "$column_header{$_}\n" } @column_index;

print "</tr>";
  
  # add sort to callback
  $callback = $form->escape($callback . "&sort=$form->{sort}");
  
  # initial item for subtotals
  $sameitem = $form->{GL}->[0]->{$form->{sort}};
  
  foreach $ref (@{ $form->{GL} }) {

    # if item ne sort print subtotal
    if ($form->{l_subtotal} eq 'Y') {
      if ($sameitem ne $ref->{$form->{sort}}) {
	&gl_subtotal;
      }
    }
    
    
    $subtotaldebit += $ref->{debit};
    $subtotalcredit += $ref->{credit};
    
    $totaldebit += $ref->{debit};
    $totalcredit += $ref->{credit};

    $ref->{debit} = $form->format_amount(\%myconfig, $ref->{debit});
    $ref->{credit} = $form->format_amount(\%myconfig, $ref->{credit});
    
    $column_data{transdate} = "<td>$ref->{transdate}</td>";
    $column_data{source} = "<td><a href=$ref->{module}.pl?action=edit&id=$ref->{id}&path=$form->{path}&login=$form->{login}&callback=$callback>$ref->{source}</td>";
    $column_data{description} = "<td>$ref->{description}</td>";
    $column_data{debit} = "<td align=right>&nbsp;$ref->{debit}</td>";
    $column_data{credit} = "<td align=right>&nbsp;$ref->{credit}</td>";
    $column_data{accno} = "<td align=right><a href=$href&accno=$ref->{accno}&callback=$callback>$ref->{accno}</a></td>";

    print "<tr>";
    map { print "$column_data{$_}\n" } @column_index;
    print "</tr>";
    
  }


  &gl_subtotal if ($form->{l_subtotal} eq 'Y');


  map { $column_data{$_} = "<td>&nbsp;</td>" } @column_index;
  
  $column_data{debit} = "<th align=right><font color=ffffff>&nbsp;".$form->format_amount(\%myconfig, $totaldebit)."</th>";
  $column_data{credit} = "<th align=right><font color=ffffff>&nbsp;".$form->format_amount(\%myconfig, $totalcredit)."</th>";
  
  print qq|
<tr bgcolor=000000>|;

  map { print "$column_data{$_}\n" } @column_index;

  print qq|</tr>
</table>

<p>
| . $locale->text('Click on "Source" to edit transaction');

}


sub gl_subtotal {
      
  $subtotaldebit = $form->format_amount(\%myconfig, $subtotaldebit);
  $subtotalcredit = $form->format_amount(\%myconfig, $subtotalcredit);
  
  map { $column_data{$_} = "<td>&nbsp;</td>" } qw(transdate source description accno);
  $column_data{debit} = "<th align=right><font color=ffffff>&nbsp;$subtotaldebit</font></td>";
  $column_data{credit} = "<th align=right><font color=ffffff>&nbsp;$subtotalcredit</font></td>";

  
  print "<tr bgcolor=333366>";
  map { print "$column_data{$_}\n" } @column_index;
  print "</tr>";

  $subtotaldebit = 0;
  $subtotalcredit = 0;

  $sameitem = $ref->{$form->{sort}};

}


sub edit {

  GL->transaction(\%myconfig, \%$form);
  
  $form->{title} = "Edit";
  
  &form_header;

  $i = 1;
  foreach $ref (@{ $form->{GL} }) {
    $form->{"accno_$i"} = $ref->{accno};
    if ($ref->{amount} < 0) {
      $form->{"debit_$i"} = $form->format_amount(\%myconfig, -$ref->{amount});
    } else {
      $form->{"credit_$i"} = ($ref->{amount} > 0) ? $form->format_amount(\%myconfig, $ref->{amount}) : "";
    }

    &form_row($i++);
  }

  &form_row($i);

  &form_footer;
}


sub update_form {

  @a = ();
  $count = 0;
  @flds = (qw(accno debit credit));

  for $i (1 .. $form->{rowcount}) {
    unless (($form->{"debit_$i"} eq "") && ($form->{"credit_$i"} eq "")) {
      # take accno apart
      ($form->{"accno_$i"}) = split(/--/, $form->{"accno_$i"});
      
      push @a, {};
      $j = $#a;
      
      map { $a[$j]->{$_} = $form->{"${_}_$i"} } @flds;
      $count++;
    }
  }

  for $i (1 .. $count) {
    $j = $i - 1;
    map { $form->{"${_}_$i"} = $a[$j]->{$_} } @flds;
  }

  for $i ($count + 1 .. $form->{rowcount}) {
    map { delete $form->{"${_}_$i"} } @flds;
  }

  $form->{rowcount} = $count;

  &display_form;
  
}


sub display_form {

  &form_header;

  $form->{rowcount}++;
  for $i (1 .. $form->{rowcount}) {
    &form_row($i);
  }

  &form_footer;

}


sub form_row {
  my $i = shift;
  
  print qq|<tr><td>
<select name="accno_$i">
|;

  foreach $item (split /\n/, $form->{chart}) {
    ($key, $value) = split /--/, $item;
    if ($key == $form->{"accno_$i"}) {
      print "<option selected>$key--$value\n";
    } else {
      print "<option>$key--$value\n";
    }
  }

  print qq|
</select>
  
<td><input name="debit_$i" size=10 value=$form->{"debit_$i"}></td>
<td><input name="credit_$i" size=10 value=$form->{"credit_$i"}></td>
</tr>
<input type=hidden name=rowcount value=$i>

|;

}


sub form_header {

  $form->{title} = $locale->text("$form->{title} General Ledger Transaction");
  
# $locale->text('Add General Ledger Transaction')
# $locale->text('Edit General Ledger Transaction')

  $form->header;

  $form->{source} =~ s/"/&quot;/g;
  $form->{description} =~ s/"/&quot;/g;
  

  print qq|
<h2>$form->{title}</h2>

<form method=post action=$form->{script}>

<input name=id type=hidden value=$form->{id}>

<input name=chart type=hidden value="$form->{chart}">

<table border=1>
<tr><td colspan=3>
<table border=0>
<tr>
<th align=right>|.$locale->text('Source').qq|</th><td><input name=source value="$form->{source}"></td>
<th align=right>|.$locale->text('Date').qq| ($myconfig{dateformat})</th><td><input name=transdate size=10 value=$form->{transdate}></td>
</tr>

<tr>
<th align=right>|.$locale->text('Description').qq|</th><td colspan=3><input name=description size=50 value="$form->{description}"></td>
</tr>

</table>
</td></tr>

<tr bgcolor=336666>
<th><font color=ffffff>|.$locale->text('Account').qq|</th>
<th><font color=ffffff>|.$locale->text('Debit').qq|</th>
<th><font color=ffffff>|.$locale->text('Credit').qq|</th>
</tr>
|;

}


sub form_footer {

  print qq|

<input type=hidden name=path value=$form->{path}>
<input type=hidden name=login value=$form->{login}>

<input name=callback type=hidden value="$form->{callback}">

</table>

<input class=submit type=submit name=action value="|.$locale->text('Update Form').qq|">
<input class=submit type=submit name=action value="|.$locale->text('Post Transaction').qq|">
|;

  if ($form->{id}) {
    print qq|<input class=submit type=submit name=action value="|.$locale->text('Delete Transaction').qq|">|;
  }

  print "</form>";
  
}


sub delete_transaction {

  $form->header;

  print qq|<form method=post action=$form->{script}>\n|;

  $form->{source} =~ s/"/&quot;/g;
  $form->{description} =~ s/"/&quot;/g;

  foreach $key (keys %$form) {
    print qq|<input type=hidden name=$key value="$form->{$key}">\n|;
  }

  print qq|
<h2><font color=red>|.$locale->text('Confirm!').qq|</font></h2>

<h4>|.$locale->text('Are you sure you want to delete Transaction').qq| $form->{source}</h4>
<p>
<input name=action class=submit type=submit value="|.$locale->text('Yes Delete Transaction').qq|">
</form>
|;

}


sub yes_delete_transaction {

  $form->isdeleted(GL->delete_transaction(\%myconfig, \%$form));

}


sub post_transaction {

  # check if there is something in source and date
  $form->isblank("source", $locale->text('Source missing!'));
  $form->isblank("transdate", $locale->text('Transaction Date missing!'));

  $rc = GL->post_transaction(\%myconfig, \%$form);

  if ($rc == -1) {
    $form->error($locale->text("Cannot have a value in both Debit and Credit"));
  }

  if ($rc == -2) {
    $form->error($locale->text("Debit and credit out of balance!"));
  }

  $form->isposted($rc);
  
}


