#!/usr/bin/perl

sub list_directory {
  my $FH = shift;
  my $directory = shift;
  $directory =~ s/\/$//;

  print "Directory listing of $directory or $config{'Default document'}\n";
  opendir (DIR, $directory) or my $error = $!;
  if ($error) {
    if ($error eq "Permission denied") {
      print LOGGER "INSERT\t$http_request{Method}\t$http_request{Object}\t$http_parameters{Host}\t403\t$peeraddress\t\n";
      &show_error_page(403, $FH);
    }
    else {
      print LOGGER "INSERT\t$http_request{Method}\t$http_request{Object}\t$http_parameters{Host}\t500\t$peeraddress\tFile access error: $error\n";
      print $error."\n";
      &show_error_page(500, $FH);
    }
  }

  my @dir_listing = readdir(DIR);
  close DIR;

  foreach $filename (@dir_listing) {
    my $absolute_path = $directory . "/" . $filename;
    if (($filename eq $config{'Default document'}) && (!(-d $absolute_path))) {
      transfer_file($FH, $directory . "/" . $config{'Default document'});
    }
  }

  my %directory_files;
  my @subdirectories;
  while (@dir_listing) {
    my $filename = shift (@dir_listing);
    my $absolute_path = $directory . "/" . $filename;
    if (-d $absolute_path) {
      push (@subdirectories, $filename);
      next;
    }
    my @temp;
    my @stats = stat($directory . "/" . $filename);
    push (@temp, $absolute_path);
    push (@temp, @stats);
    $directory_files{$filename} = \@temp;
  }

  unless ($http_request{Method} eq "HEAD") {
    print $FH "$http_request{Version} 200 OK\r\n";
    print $FH "Server: $config{Version}\r\n";
    print $FH "Content-type: text/html\r\n";
    print $FH "\r\n";

    print $FH "<html>\n";
    print $FH "  <head>\n";
    print $FH "    <title>Listing of $http_request{Object}</title>\n";
    if (@stylesheet) {
      print $FH "    <style type=\"text/css\">\n";
      print $FH "      $_" foreach (@stylesheet);
      print $FH "    </style>\n";
    }
    print $FH "  </head>\n";
    print $FH "  <body>\n";
    print $FH "    <h1>Listing of $http_request{Object}</h1><hr>\n";
    print $FH "    <table rows=4 width=\"100%\">\n";
    print $FH "      <tr>\n";
    print $FH "        <td class=\"list-heading\" width=24></td>\n";
    print $FH "        <td class=\"list-heading\">Filename</td>\n";
    print $FH "        <td class=\"list-heading\" width=\"15%\"><p align=\"right\">Size</td></p>\n";
    print $FH "        <td class=\"list-heading\" width=\"20%\"><p align=\"right\">Last modified</p></td>\n";
    print $FH "      </tr>\n";

    foreach $filename (sort @subdirectories) {
      print $FH "      <tr>\n";
      print $FH "        <td class=\"list-body\" height=24><img src=\"/icons/folder.gif\"></td>\n";
      print $FH "        <td class=\"list-body\"><a href=\"$filename/\">$filename</a></td>\n";
      print $FH "        <td class=\"list-body\"><p align=\"right\">Directory</p></td>\n";
      print $FH "        <td class=\"list-body\"></td>\n";
      print $FH "      </tr>\n";
    }

    foreach $filename (sort keys %directory_files) {
      next if (($http_request{Object} eq "/") && ($filename eq ".."));
      my @file_info = @{$directory_files{$filename}};
      my $href = $filename;
      my $extension = (reverse(split(/\./, $filename)))[0];
      my $iconfile = "/icons/$icons{$extension}" if ($icons{$extension});
      my $size;
      
      if ($file_info[8] < 1024) {
	$size = "$file_info[8] bytes";
      }
      elsif ($file_info[8] < 1048576) {
	$size = sprintf("%.2f", ($file_info[8] / 1024)) . " kB";
      }
      else {
	$size = sprintf("%.2f", ($file_info[8] / 1048576)) . " MB";
      }
      my $timestamp = strftime("%a %d %b %C, %H:%M:%S", gmtime($file_info[11]));

      print $FH "      <tr>\n";
      print $FH "        <td class=\"list-body\">";
      if ($iconfile) {
	print $FH "<img src=\"$iconfile\">";
      }
      print $FH "</td>\n";
      print $FH "        <td class=\"list-body\" height=24><a href=\"$href\">$filename</a></td>\n";
      print $FH "        <td class=\"list-body\"><p align=\"right\">$size</td></p>\n";
      print $FH "        <td class=\"list-body\"><p align=\"right\">$timestamp</td></p>\n";
      print $FH "      </tr>\n";
    }

    print $FH "    </table>\n";
    print $FH "    <br><hr>\n";
    print $FH "    <i>$config{Version} at "; 
    if ($http_parameters{Host}) {
      print $FH $http_parameters{Host};
    }
    else {
      print $FH $config{'HTTP Host'};
    }
    print $FH " ($config{os}, $config{hardware})</p>\n";
    print $FH "  </body>\n";
    print $FH "</html>\n";
  }
  close $FH;
  print LOGGER "INSERT\t$http_request{Method}\t$http_request{Object}\t$http_parameters{Host}\t200\t$peeraddress\t\n";
  exit;
}


return 1;
