#!/usr/bin/perl
#
# DNHTTPD main file
# This is the script you should execute to start the server.
#
# Copyright: Doug Neal, 2001
# www.dougneal.co.uk
#

use Socket;
use POSIX qw (strftime);
use IPC::Open2;

require "./http.pl";
require "./errors.pl";
require "./configuration.pl";
require "./dirlist.pl";
require "./handlers.pl";
require "./fileio.pl";

#Change this to wherever you installed DNHTTPD.
$config{SERVER_ROOT} = "..";
###############################################

print STDERR "--> DNHTTPD starting up!\n";

$SIG{HUP} = \&recompile;
$SIG{CHLD} = IGNORE;

&read_configuration;
&load_errors;
&bind_socket;
&ready_mode;

sub read_configuration {
  &config_main;
  &config_system;
  &config_vhosts;
  &config_mime;
  &config_stylesheet;
  &config_icons;
}

if ($config{Logging} eq "Yes") {
  &log_initialise;
}

sub log_initialise {
  if ($config{Logging} eq "Yes") {
    if (!open(LOGGER, "|./logger.pl $config{SERVER_ROOT}/conf/logger.conf")) {
      print STDERR "  --> Fatal error starting logger: $!\n";
      exit;
    }
    else {
      print STDERR "  --> Logging initialised\n";
    }
  }
}

sub bind_socket {
  socket(SERVERSOCK, PF_INET, SOCK_STREAM, getprotobyname('tcp')) || die "Socket: $!\n";
  bind(SERVERSOCK, sockaddr_in($config{'HTTP Port'}, INADDR_ANY)) || die "Bind: $!\n";
  listen(SERVERSOCK, 1) || die "Listen: $!\n";
}

sub ready_mode {
  print STDERR "--> DNHTTPD is ready and waiting for connections.\n";

  while (1) {
    ($peerport, $iaddr) = sockaddr_in(accept(FD, SERVERSOCK));
    $peeraddress = inet_ntoa($iaddr);
    my $pid = fork();
    if (!$pid) {
      serve(*FD, $peeraddress);
    }
    else {
      print "\nProcessing request from $peeraddress:$peerport\n";
      close (FD);
    }
  }
}

sub recompile {
  print STDERR "\n\n *** SIGHUP received\n";
  print STDERR " *** Recompiling server into memory\n";
  eval(`cat configuration.pl`); print STDERR $@;
  eval(`cat dirlist.pl`); print STDERR $@;
  eval(`cat errors.pl`); print STDERR $@;
  eval(`cat fileio.pl`); print STDERR $@;
  eval(`cat handlers.pl`); print STDERR $@;
  eval(`cat http.pl`); print STDERR $@;
  print STDERR " *** Recompilation complete\n";
  print STDERR " *** Re-reading configuration files\n";
  &read_configuration;
  print STDERR "\n *** Server restart completed\n\n";
  $SIG{HUP} = \&recompile;
}
