#!/usr/bin/perl

$bindir = "../../bin/mozilla";


opendir DIR, "$bindir" || die "$!";
@allfiles = grep /\.pl/, readdir DIR;
closedir DIR;

# slurp the translations in
if (-f "all") {
  require "all";
}

# load copyright
if (-f "copyright") {
  open FH, "copyright";
} else {
  open FH, "../../doc/copyright";
}
@copyrightdefault = <FH>;
close FH;


foreach $file (@allfiles) {
  
  %locale = ();
  %submit = ();
  %subrt = ();
  
  open FH, "$bindir/$file" || die "$!";
  
  $file =~ s/\.pl//;
  
  while (<FH>) {

    # is this a sub ?
    if (/^sub /) {
      ($null, $subrt) = split / +/;
      $subrt{$subrt} = 1;
      next;
    }
    
    $rc = 1;
    
    while ($rc) {
      if (/Locale/) {
	unless (/^use /) {
	  ($null, $country) = split /,/;
	  $country =~ s/^ +["']//;
	  $country =~ s/["'].*//;
	}
      }

      if (/\$locale->text.*?\)/) {
	$string = $&;
	$string =~ s/\$locale->text\(\s*['"(q|qq)]//;
	$string =~ s/\D\s*\)$//;

        # if there is no $form in the string record it
	unless ($string =~ /\$form.*/) {
	  # this guarantees one instance of string
	  $locale{$string} = 1;

          # this one is for all the locales
	  $alllocales{$string} = 1;

          # is it a submit button before $locale->
          if (/type=submit/) {
	    $submit{$string} = 1;
          }
	}
      }

      # exit loop if there are no more locales on this line
      ($rc) = ($' =~ /\$locale->text/);
      # strip text
      s/^.*?\$locale->text.*?\)//;
    }
  }

  close FH;


  @copyright = ();
  if (-f "$file") {
    open FH, "$file" || die "$!";
    while (<FH>) {
      s/
//;
      push @copyright, $_;
      last if (/^$/);
    }
    close FH;
  }


  if (-f "missing") {
    require 'missing';
  }
  unlink 'missing';

  foreach $text (keys %$missing) {
    unless ($self{texts}{$text}) {
      $locale{$text} = 1;
      $self{texts}{$text} = $missing->{$text};
    }
  }


  open FH, ">$file" || die "$!";

  # print copyright
  if (@copyright) {
    print FH @copyright;
  } else {
    print FH @copyrightdefault;
  }

  print FH q|
$self{texts} = {
|;

  foreach $key (sort keys %locale) {
    if ($self{texts}{$key}) {
      $text = $self{texts}{$key};
    } else {
      $text = $key;
    }
    $text =~ s/'/\\'/g;
    $text =~ s/\\$/\\\\/;

    $keytext = $key;
    $keytext =~ s/'/\\'/g;
    $keytext =~ s/\\$/\\\\/;
    
    print FH qq|  '$keytext'|.(' ' x (27-length($keytext))).qq| => '$text',\n|;
  }

  print FH q|};

$self{subs} = {
|;
  
  foreach $key (sort keys %subrt) {
    $text = $key;
    $text =~ s/'/\\'/g;
    $text =~ s/\\$/\\\\/;
    print FH qq|  '$text'|.(' ' x (27-length($text))).qq| => '$text',\n|;
  }

  foreach $key (sort keys %submit) {
    $text = ($self{texts}{$key}) ? $self{texts}{$key} : $key;
    $text =~ s/'/\\'/g;
    $text =~ s/\\$/\\\\/;

    $english_sub = $key;
    $english_sub =~ s/'/\\'/g;
    $english_sub =~ s/\\$/\\\\/;
    $english_sub = lc $key;
    
    $translated_sub = lc $text;
    $english_sub =~ s/( |-)/_/g;
    $translated_sub =~ s/( |-)/_/g;
    print FH qq|  '$translated_sub'|.(' ' x (27-length($translated_sub))).qq| => '$english_sub',\n|;
  }
  
  print FH q|};

1;
|;

  close FH;
}


# now print out all

open FH, ">all" || die "$!";

print FH q|# These are all the texts to build the translations files.
# The file has the form of
# 'english text'  => 'foreign text',
# you can add the translation in this file or in the 'missing' file
# When you add/change the texts run locales.pl from this directory
# to rebuild the translation files

$self{texts} = {
|;


foreach $key (sort keys %alllocales) {
  $text = $self{texts}{$key};

  $count++;
  
  $text =~ s/'/\\'/g;
  $text =~ s/\\$/\\\\/;
  $key =~ s/'/\\'/g;
  $key =~ s/\\$/\\\\/;

  unless ($text) {
    $notext++;
    push @missing, $key;
  }

  print FH qq|  '$key'|.(' ' x (27-length($key))).qq| => '$text',\n|;

}

print FH q|};

1;
|;

close FH;


if (@missing) {
  open FH, ">missing" || die "$!";

  print FH q|# add the missing texts and run locales.pl to rebuild

$missing = {
|;

  foreach $text (@missing) {
    print FH qq|  '$text'|.(' ' x (27-length($text))).qq| => '',\n|;
  }

  print FH q|};

1;
|;

  close FH;
}


$per = sprintf("%.1f", ($count - $notext) / $count * 100);
print "${per}% done\n";

