-- Oracle-tables.sql
--
ALTER SESSION SET NLS_DATE_FORMAT = 'YYYY-MM-DD';
SET SCAN OFF;
--
CREATE SEQUENCE id START WITH 10000 INCREMENT BY 1 MAXVALUE 2147483647 MINVALUE 1  CACHE 2;
SELECT ID.NEXTVAL FROM DUAL;
--
CREATE TABLE makemodel (
  id INTEGER,
  parts_id INTEGER,
  name LONG
);
--
CREATE TABLE gl (
  id INTEGER,
  source VARCHAR2(30),
  description LONG,
  transdate DATE DEFAULT SYSDATE
);
--
CREATE TABLE chart (
  id INTEGER,
  accno INTEGER UNIQUE,
  description VARCHAR2(100),
  charttype CHAR(1) DEFAULT 'A',
  gifi INTEGER,
  category CHAR(1),
  link LONG
);
--
CREATE TABLE defaults (
  inventory_accno_id INTEGER,
  income_accno_id INTEGER,
  expense_accno_id INTEGER,
  invnumber VARCHAR2(30),
  ponumber VARCHAR2(30),
  yearend VARCHAR2(5),
  nativecurr VARCHAR2(3),
  weightunit VARCHAR2(5),
  businessnumber VARCHAR2(30),
  version VARCHAR2(8)
);
INSERT INTO defaults (version) VALUES ('1.6.0');
--
CREATE TABLE acc_trans (
  trans_id INTEGER,
  accno INTEGER,
  amount FLOAT,
  transdate DATE DEFAULT SYSDATE
);
--
CREATE TABLE invoice (
  id INTEGER,
  trans_id INTEGER,
  inventory_accno_id INTEGER,
  income_accno_id INTEGER,
  expense_accno_id INTEGER,
  parts_id INTEGER,
  partnumber VARCHAR2(30), 
  description LONG,
  qty FLOAT,
  allocated FLOAT,
  sellprice FLOAT,
  discount FLOAT,
  unit VARCHAR2(5),
  assemblyitem CHAR(1)
);
--
CREATE TABLE vendor (
  id INTEGER,
  name VARCHAR2(35),
  addr1 VARCHAR2(35),
  addr2 VARCHAR2(35),
  addr3 VARCHAR2(35),
  contact VARCHAR2(35),
  phone VARCHAR2(20),
  fax VARCHAR2(20),
  email VARCHAR2(50),
  notes LONG,
  terms INTEGER DEFAULT 0,
  taxincluded CHAR(1)
);
--
CREATE TABLE customer (
  id INTEGER,
  name VARCHAR2(35),
  addr1 VARCHAR2(35),
  addr2 VARCHAR2(35),
  addr3 VARCHAR2(35),
  contact VARCHAR2(35),
  phone VARCHAR2(20),
  fax VARCHAR2(20),
  email VARCHAR2(50),
  notes LONG,
  discount FLOAT,
  taxincluded CHAR(1),
  creditlimit FLOAT,
  terms INTEGER DEFAULT 0,
  shiptoname VARCHAR2(35),
  shiptoaddr1 VARCHAR2(35),
  shiptoaddr2 VARCHAR2(35),
  shiptoaddr3 VARCHAR2(35),
  shiptocontact VARCHAR2(20),
  shiptophone VARCHAR2(20),
  shiptofax VARCHAR2(20),
  shiptoemail VARCHAR2(50)
);
--
CREATE TABLE parts (
  id INTEGER,
  partnumber VARCHAR2(30), 
  description LONG,
  unit VARCHAR2(5),
  listprice FLOAT,
  sellprice FLOAT,
  lastcost FLOAT,
  priceupdate DATE DEFAULT SYSDATE,
  weight FLOAT,
  onhand FLOAT,
  notes VARCHAR2(2000),
  makemodel CHAR(1) DEFAULT 'f',
  assembly CHAR(1) DEFAULT 'f',
  alternate CHAR(1) DEFAULT 'f',
  rop FLOAT,
  inventory_accno_id INTEGER DEFAULT 0,
  income_accno_id INTEGER,
  expense_accno_id INTEGER
);
--
CREATE TABLE ar (
  id INTEGER,
  invnumber VARCHAR2(30),
  transdate DATE DEFAULT SYSDATE,
  customer INTEGER,
  taxincluded CHAR(1),
  amount FLOAT,
  netamount FLOAT,
  paid FLOAT,
  datepaid DATE,
  duedate DATE,
  invoice CHAR(1) DEFAULT 'f',
  shippingpoint VARCHAR2(100),
  terms INTEGER DEFAULT 0,
  notes LONG
);
--
CREATE TABLE ap (
  id INTEGER,
  invnumber VARCHAR2(30),
  transdate DATE DEFAULT SYSDATE,
  vendor INTEGER,
  taxincluded CHAR(1),
  amount FLOAT,
  netamount FLOAT,
  paid FLOAT,
  datepaid DATE,
  duedate DATE,
  invoice CHAR(1) DEFAULT 'f',
  ordnumber VARCHAR(30)
);
--
CREATE TABLE partstax (
  parts_id INTEGER,
  chart_id INTEGER
);
--
CREATE TABLE tax (
  chart_id INTEGER,
  rate FLOAT,
  taxnumber VARCHAR2(30)
);
--
CREATE TABLE customertax (
  customer_id INTEGER,
  chart_id INTEGER
);
--
CREATE TABLE vendortax (
  vendor_id INTEGER,
  chart_id INTEGER
);
